=== modified file 'drizzled/Makefile.am'
--- drizzled/Makefile.am	2009-01-24 16:18:44 +0000
+++ drizzled/Makefile.am	2009-01-28 02:39:29 +0000
@@ -41,6 +41,7 @@
 		authentication.h \
 		base.h \
 		cached_item.h \
+		calendar.h \
 		check_stack_overrun.h \
 		client_settings.h \
 		current_session.h \
@@ -323,6 +324,8 @@
 		table_map_iterator.h \
 		table_share.h \
 		tableop_hooks.h \
+		temporal.h \
+		temporal_format.h \
 		tmp_table.h \
 		tzfile.h \
 		tztime.h \
@@ -347,7 +350,7 @@
 		 $(top_builddir)/drizzled/serialize/libserialize.la \
 		 @mysql_plugin_libs@ \
 		 $(LDADD) $(LTLIBUUID) \
-		 $(LIBDL_LIBS) $(LTLIBEVENT) $(LTLIBPROTOBUF) \
+		 $(LIBDL_LIBS) $(LTLIBEVENT) $(LTLIBPROTOBUF) $(PCRE_LIBS) \
 		 $(DRIZZLED_PLUGIN_DEP_LIBS)
 drizzled_DEPENDENCIES = @mysql_plugin_libs@ ${noinst_LTLIBRARIES}
 
@@ -366,6 +369,7 @@
 drizzled_SOURCES = \
 		authentication.cc \
 		cached_item.cc \
+		calendar.cc \
 		check_stack_overrun.cc \
 		comp_creator.cc \
 		configvar.cc \
@@ -437,6 +441,8 @@
 		table.cc \
                 table_map_iterator.cc \
 		tableop_hooks.cc \
+		temporal.cc \
+		temporal_format.cc \
 		thr_malloc.cc \
 		time.cc \
 		tmp_table.cc \

=== added file 'drizzled/calendar.cc'
--- drizzled/calendar.cc	1970-01-01 00:00:00 +0000
+++ drizzled/calendar.cc	2009-01-28 02:39:29 +0000
@@ -0,0 +1,489 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *
+ * Common functions for dealing with calendrical calculations
+ */
+
+#include "drizzled/global.h"
+#include "drizzled/calendar.h"
+
+/** Static arrays for number of days in a month and their "day ends" */
+static const uint32_t __leap_days_in_month[12]=       {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
+static const uint32_t __normal_days_in_month[12]=     {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
+static const uint32_t __leap_days_to_end_month[13]=   {0, 31, 60, 91, 121, 151, 182, 213, 244, 274, 305, 335, 366};
+static const uint32_t __normal_days_to_end_month[13]= {0, 31, 59, 90, 120, 150, 181, 212, 243, 273, 304, 334, 365};
+
+/** 
+ * Private utility macro for enabling a switch between
+ * Gregorian and Julian leap year date arrays.
+ */
+#define __DAYS_IN_MONTH(y, c) (const uint32_t *) (IS_LEAP_YEAR((y),(c)) ? __leap_days_in_month : __normal_days_in_month)
+#define __DAYS_TO_END_MONTH(y, c) (const uint32_t *) (IS_LEAP_YEAR((y),(c)) ? __leap_days_to_end_month : __normal_days_to_end_month)
+
+/**
+ * Calculates the Julian Day Number from the year, month 
+ * and day at noon supplied in the Julian calendar.
+ *
+ * The following formula is used to calculate the Julian
+ * Day Number from a date in the Julian Calendar.
+ *
+ * The months January to December are 1 to 12. 
+ * Astronomical year numbering is used, thus 1 BC is 0, 2 BC is −1, 
+ * and 4713 BC is −4712. In all divisions (except for JD) the floor 
+ * function is applied to the quotient (for dates since 
+ * March 1, −4800 all quotients are non-negative, so we can also 
+ * apply truncation).
+ *
+ * a = (14 - month) / 12
+ * y = year + 4800 - a
+ * m = month + 12a - 3
+ * JDN = day + ((153m + 2) / 5) + 365y + (y / 4) - 32083
+ *
+ * @cite http://en.wikipedia.org/wiki/Julian_day#Calculation
+ *
+ * @note
+ *
+ * Year month and day values are assumed to be valid.  This 
+ * method does no bounds checking or validation.
+ *
+ * @param Year of date
+ * @param Month of date
+ * @param Day of date
+ */
+int64_t julian_day_number_from_julian_date(uint32_t year, uint32_t month, uint32_t day)
+{
+  int64_t day_number;
+  int64_t a= (14 - month) / 12;
+  int64_t y= year + 4800 - a;
+  int64_t m= month + (12 * a) - 3;
+
+  day_number= day + (((153 * m) + 2) / 5) + (365 * y) + (y / 4) - 32083;
+  return day_number;
+}
+
+/**
+ * Calculates the Julian Day Number from the year, month 
+ * and day supplied.  The calendar used by the supplied
+ * year, month, and day is assumed to be Gregorian Proleptic.
+ *
+ * The months January to December are 1 to 12. 
+ * Astronomical year numbering is used, thus 1 BC is 0, 2 BC is −1, 
+ * and 4713 BC is −4712. In all divisions (except for JD) the floor 
+ * function is applied to the quotient (for dates since 
+ * March 1, −4800 all quotients are non-negative, so we can also 
+ * apply truncation).
+ *
+ * a = (14 - month) / 12
+ * y = year + 4800 - a
+ * m = month + 12a - 3
+ * JDN = day + ((153m + 2) / 5) + 365y + (y / 4) - (y / 100) + (y / 400) - 32045
+ *
+ * @cite http://en.wikipedia.org/wiki/Julian_day#Calculation
+ *
+ * @note
+ *
+ * Year month and day values are assumed to be valid.  This 
+ * method does no bounds checking or validation.
+ *
+ * @param Year of date
+ * @param Month of date
+ * @param Day of date
+ */
+int64_t julian_day_number_from_gregorian_date(uint32_t year, uint32_t month, uint32_t day)
+{
+  int64_t day_number;
+  int64_t a= (14 - month) / 12;
+  int64_t y= year + 4800 - a;
+  int64_t m= month + (12 * a) - 3;
+
+  day_number= day + (((153 * m) + 2) / 5) + (365 * y) + (y / 4) - (y / 100) + (y / 400) - 32045;
+  return day_number;
+}
+
+/**
+ * Translates an absolute day number to a 
+ * Julian day number.  Note that a Julian day number
+ * is not the same as a date in the Julian proleptic calendar.
+ *
+ * @param The absolute day number
+ */
+int64_t absolute_day_number_to_julian_day_number(int64_t absolute_day)
+{
+  return absolute_day + JULIAN_DAY_NUMBER_AT_ABSOLUTE_DAY_ONE;
+}
+
+/**
+ * Translates a Julian day number to an 
+ * absolute day number.  Note that a Julian day number
+ * is not the same as a date in the Julian proleptic calendar.
+ *
+ * @param The Julian day number
+ */
+int64_t julian_day_number_to_absolute_day_number(int64_t julian_day)
+{
+  return julian_day - JULIAN_DAY_NUMBER_AT_ABSOLUTE_DAY_ONE;
+}
+
+/**
+ * Given a supplied Julian Day Number, populates a year, month, and day
+ * with the date in the Gregorian Proleptic calendar which corresponds to
+ * the given Julian Day Number.
+ *
+ * @cite Algorithm from http://en.wikipedia.org/wiki/Julian_day
+ *
+ * @param Julian Day Number
+ * @param Pointer to year to populate
+ * @param Pointer to month to populate
+ * @param Pointer to the day to populate
+ */
+void gregorian_date_from_julian_day_number(int64_t julian_day
+                                         , uint32_t *year_out
+                                         , uint32_t *month_out
+                                         , uint32_t *day_out)
+{
+  int64_t j = julian_day + 32044;
+  int64_t g = j / 146097;
+  int64_t dg = j % 146097;
+  int64_t c = (dg / 36524 + 1) * 3 / 4;
+  int64_t dc = dg - c * 36524;
+  int64_t b = dc / 1461;
+  int64_t db = dc % 1461;
+  int64_t a = (db / 365 + 1) * 3 / 4;
+  int64_t da = db - a * 365;
+  int64_t y = g * 400 + c * 100 + b * 4 + a;
+  int64_t m = (da * 5 + 308) / 153 - 2;
+  int64_t d = da - (m + 4) * 153 / 5 + 122;
+  int64_t Y = y - 4800 + (m + 2) / 12;
+  int64_t M = (m + 2) % 12 + 1;
+  int64_t D = d + 1.5;
+
+  /* Push out parameters */
+  *year_out= (uint32_t) Y;
+  *month_out= (uint32_t) M;
+  *day_out= (uint32_t) D;
+}
+
+/**
+ * Given a supplied Absolute Day Number, populates a year, month, and day
+ * with the date in the Gregorian Proleptic calendar which corresponds to
+ * the given Absolute Day Number.
+ *
+ * @param Absolute Day Number
+ * @param Pointer to year to populate
+ * @param Pointer to month to populate
+ * @param Pointer to the day to populate
+ */
+void gregorian_date_from_absolute_day_number(int64_t absolute_day
+                                           , uint32_t *year_out
+                                           , uint32_t *month_out
+                                           , uint32_t *day_out)
+{
+  gregorian_date_from_julian_day_number(
+      absolute_day_number_to_julian_day_number(absolute_day)
+    , year_out
+    , month_out
+    , day_out);
+}
+
+/**
+ * Functions to calculate the number of days in a 
+ * particular year.  The number of days in a year 
+ * depends on the calendar used for the date.
+ *
+ * For the Julian proleptic calendar, a leap year 
+ * is one which is evenly divisible by 4.
+ *
+ * For the Gregorian proleptic calendar, a leap year
+ * is one which is evenly divisible by 4, and if
+ * the year is evenly divisible by 100, it must also be evenly
+ * divisible by 400.
+ */
+
+/**
+ * Returns the number of days in a particular year
+ * depending on the supplied calendar.
+ *
+ * @param year to evaluate
+ * @param calendar to use
+ */
+inline uint32_t days_in_year(const uint32_t year, enum calendar calendar)
+{
+  if (calendar == GREGORIAN)
+    return days_in_year_gregorian(year);
+  return days_in_year_julian(year);
+}
+
+/**
+ * Returns the number of days in a particular Julian calendar year.
+ *
+ * @param year to evaluate
+ */
+inline uint32_t days_in_year_julian(const uint32_t year)
+{
+  /* Short-circuit. No odd years can be leap years... */
+  return (year & 3) == 0;
+}
+
+/**
+ * Returns the number of days in a particular Gregorian year.
+ *
+ * @param year to evaluate
+ */
+inline uint32_t days_in_year_gregorian(const uint32_t year)
+{
+  /* Short-circuit. No odd years can be leap years... */
+  if ((year & 1) == 1)
+    return 365;
+  return (            
+            (year & 3) == 0 
+            && (year % 100 || ((year % 400 == 0) && year)) 
+            ? 366 
+            : 365
+         );
+}
+
+/**
+ * Returns the number of the day in a week.
+ *
+ * Return values:
+ *
+ * Day            Day Number  Sunday first day?
+ * -------------- ----------- -----------------
+ * Sunday         0           true
+ * Monday         1           true
+ * Tuesday        2           true
+ * Wednesday      3           true
+ * Thursday       4           true
+ * Friday         5           true
+ * Saturday       6           true
+ * Sunday         6           false
+ * Monday         0           false
+ * Tuesday        1           false
+ * Wednesday      2           false
+ * Thursday       3           false
+ * Friday         4           false
+ * Saturday       5           false
+ *
+ * @param Julian Day Number
+ * @param Consider Sunday the first day of the week?
+ */
+uint32_t day_of_week(int64_t day_number
+                   , bool sunday_is_first_day_of_week)
+{
+  uint32_t tmp= (uint32_t) (day_number % 7);
+  /* 0 returned from above modulo is a Monday */
+  if (sunday_is_first_day_of_week)
+    tmp= (tmp == 6 ? 0 : tmp + 1);
+  return tmp;
+}
+
+/**
+ * Given a year, month, and day, returns whether the date is 
+ * valid for the Gregorian proleptic calendar.
+ *
+ * @param The year
+ * @param The month
+ * @param The day
+ */
+bool is_valid_gregorian_date(uint32_t year, uint32_t month, uint32_t day)
+{
+  if (year < 1)
+    return false;
+  if (month != 2)
+    return (day <= __normal_days_in_month[month - 1]);
+  else
+  {
+    const uint32_t *p_months= __DAYS_IN_MONTH(year, (enum calendar) GREGORIAN);
+    return (day <= p_months[1]);
+  }
+}
+
+/**
+ * Returns the number of days in a month, given
+ * a year and a month in the Gregorian calendar.
+ *
+ * @param Year in Gregorian Proleptic calendar
+ * @param Month in date
+ */
+uint32_t days_in_gregorian_year_month(uint32_t year, uint32_t month)
+{
+  const uint32_t *p_months= __DAYS_IN_MONTH(year, GREGORIAN);
+  return p_months[month - 1];
+}
+
+/**
+ * Returns whether the supplied date components are within the 
+ * range of the UNIX epoch.
+ *
+ * Times in the range of 1970-01-01T00:00:00 to 2038-01-19T03:14:07
+ *
+ * @param Year
+ * @param Month
+ * @param Day
+ * @param Hour
+ * @param Minute
+ * @param Second
+ */
+bool in_unix_epoch_range(uint32_t year
+                       , uint32_t month
+                       , uint32_t day
+                       , uint32_t hour
+                       , uint32_t minute
+                       , uint32_t second)
+{
+  if (month == 0 || day == 0)
+    return false;
+  if (year < UNIX_EPOCH_MAX_YEARS
+      && year >= UNIX_EPOCH_MIN_YEARS)
+    return true;
+  if (year < UNIX_EPOCH_MIN_YEARS)
+    return false;
+  if (year == UNIX_EPOCH_MAX_YEARS)
+  {
+    if (month > 1)
+      return false;
+    if (day > 19)
+      return false;
+    else if (day < 19)
+      return true;
+    else
+    {
+      /* We are on the final day of UNIX Epoch */
+      uint32_t seconds= (hour * 60 * 60)
+                      + (minute * 60)
+                      + (second);
+      if (seconds <= ((3 * 60 * 60) + (14 * 60) + 7))
+        return true;
+      return false;
+    }
+  }
+  return false;
+}
+
+/**
+ * Returns the number of the week from a supplied year, month, and
+ * date in the Gregorian proleptic calendar.
+ *
+ * The week number returned will depend on the values of the
+ * various boolean flags passed to the function.
+ *
+ * The flags influence returned values in the following ways:
+ *
+ * sunday_is_first_day_of_week
+ *
+ * If TRUE, Sunday is first day of week
+ * If FALSE,	Monday is first day of week
+ *
+ * week_range_is_ordinal
+ *
+ * If FALSE, the week is in range 0-53
+ *
+ * Week 0 is returned for the the last week of the previous year (for
+ * a date at start of january) In this case one can get 53 for the
+ * first week of next year.  This flag ensures that the week is
+ * relevant for the given year. 
+ *
+ * If TRUE, the week is in range 1-53.
+ *
+ * In this case one may get week 53 for a date in January (when
+ * the week is that last week of previous year) and week 1 for a
+ * date in December.
+ *
+ * use_iso_8601_1988
+ *
+ * If TRUE, the weeks are numbered according to ISO 8601:1988
+ *
+ * ISO 8601:1988 means that if the week containing January 1 has
+ * four or more days in the new year, then it is week 1;
+ * Otherwise it is the last week of the previous year, and the
+ * next week is week 1.
+ *
+ * If FALSE, the week that contains the first 'first-day-of-week' is week 1.
+ *
+ * @param Subject year
+ * @param Subject month
+ * @param Subject day
+ * @param Is sunday the first day of the week?
+ * @param Is the week range ordinal?
+ * @param Should we use ISO 8601:1988 rules?
+ * @param Pointer to a uint32_t to hold the resulting year, which 
+ *        may be incremented or decremented depending on flags
+ */
+uint32_t week_number_from_gregorian_date(uint32_t year
+                                       , uint32_t month
+                                       , uint32_t day
+                                       , bool sunday_is_first_day_of_week
+                                       , bool week_range_is_ordinal
+                                       , bool use_iso_8601_1988
+                                       , uint32_t *year_out)
+{
+  uint32_t tmp_days;
+  uint32_t day_number= julian_day_number_from_gregorian_date(year, month, day);
+  uint32_t first_day_of_year= julian_day_number_from_gregorian_date(year, 1, 1);
+  uint32_t tmp_years= year;
+
+  uint32_t week_day= day_of_week(first_day_of_year, sunday_is_first_day_of_week);
+
+  if (month == 1 && day <= (7 - week_day))
+  {
+    if (
+        (! week_range_is_ordinal) 
+        && 
+        ( (use_iso_8601_1988 && week_day != 0) 
+          || (!use_iso_8601_1988 && (week_day >= 4))
+        )
+    )
+    {
+      if (year_out != NULL)
+        *year_out= tmp_years;
+      return 0;
+    }
+    week_range_is_ordinal= true;
+    tmp_years--;
+    tmp_days= days_in_year(tmp_years, GREGORIAN);
+    first_day_of_year-= tmp_days;
+    week_day= (week_day + (53 * 7) - tmp_days) % 7;
+  }
+
+  if ((!use_iso_8601_1988 && week_day != 0) 
+      || (use_iso_8601_1988 && week_day >= 4))
+    tmp_days= day_number - (first_day_of_year + (7 - week_day));
+  else
+    tmp_days= day_number - (first_day_of_year - week_day);
+
+  if (week_range_is_ordinal && tmp_days >= (52 * 7))
+  {
+    week_day= (week_day + days_in_year(tmp_years, GREGORIAN)) % 7;
+    if ((!use_iso_8601_1988 && week_day < 4) 
+        || (use_iso_8601_1988 && week_day == 0))
+    {
+      tmp_years++;
+      if (year_out != NULL)
+        *year_out= tmp_years;
+      return 1;
+    }
+  }
+  if (year_out != NULL)
+    *year_out= tmp_years;
+  return (tmp_days / 7) + 1;
+}

=== added file 'drizzled/calendar.h'
--- drizzled/calendar.h	1970-01-01 00:00:00 +0000
+++ drizzled/calendar.h	2009-01-28 02:39:29 +0000
@@ -0,0 +1,346 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ * 
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *
+ * Structures and functions for:
+ *
+ * Calculating day number in Gregorian and Julian proleptic calendars.
+ * Converting between day numbers and dates in the calendars.
+ * Converting between different calendars.
+ * Calculating differences between dates.
+ *
+ * Works used in research:
+ *
+ * @cite "Calendrical Calculations", Dershowitz and Reingold
+ * @cite ISO 8601 http://en.wikipedia.org/wiki/ISO_8601
+ * @cite http://www.ddj.com/hpc-high-performance-computing/197006254 
+ * @cite http://en.wikipedia.org/wiki/Julian_day#Calculation
+ */
+
+#ifndef DRIZZLED_CALENDAR_H
+#define DRIZZLED_CALENDAR_H
+
+#define JULIAN_DAY_NUMBER_AT_ABSOLUTE_DAY_ONE INT64_C(1721425)
+
+#define DAYS_IN_NORMAL_YEAR INT32_C(365)
+#define DAYS_IN_LEAP_YEAR INT32_C(366)
+
+#define GREGORIAN_START_YEAR 1582
+#define GREGORIAN_START_MONTH 10
+#define GREGORIAN_START_DAY 15
+
+#define UNIX_EPOCH_MAX_YEARS 2038
+#define UNIX_EPOCH_MIN_YEARS 1970
+
+/**
+ * The following constants define the system of calculating the number
+ * of days in various periods of time in the Gregorian calendar.
+ *
+ * Leap years (years containing 366 days) occur:
+ *
+ * - When the year is evenly divisible by 4
+ * - If the year is evenly divisible by 100, it must also
+ *   be evenly divisible by 400.
+ */
+#define GREGORIAN_DAYS_IN_400_YEARS UINT32_C(146097)
+#define GREGORIAN_DAYS_IN_100_YEARS UINT32_C(36524)
+#define GREGORIAN_DAYS_IN_4_YEARS   UINT32_C(1461)
+
+/**
+ * Simple macro returning whether the supplied year
+ * is a leap year in the supplied calendar.
+ *
+ * @param Year to evaluate
+ * @param Calendar to use
+ */
+#define IS_LEAP_YEAR(y, c) (days_in_year((y),(c)) == 366)
+
+/**
+ * Simple macro returning whether the supplied year
+ * is a leap year in the Gregorian proleptic calendar.
+ */
+#define IS_GREGORIAN_LEAP_YEAR(y) (days_in_year_gregorian((y)) == 366)
+
+/**
+ * Simple macro returning whether the supplied year
+ * is a leap year in the Julian proleptic calendar.
+ */
+#define IS_JULIAN_LEAP_YEAR(y) (days_in_year_julian((y)) == 366)
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/**
+ * Different calendars supported by the temporal library
+ */
+enum calendar
+{
+  GREGORIAN= 1
+, JULIAN= 2
+, HEBREW= 3
+, ISLAM= 4
+};
+
+/**
+ * Calculates the Julian Day Number from the year, month 
+ * and day supplied for a Gregorian Proleptic calendar date.
+ *
+ * @note
+ *
+ * Year month and day values are assumed to be valid.  This 
+ * method does no bounds checking or validation.
+ *
+ * @param Year of date
+ * @param Month of date
+ * @param Day of date
+ */
+int64_t julian_day_number_from_gregorian_date(uint32_t year, uint32_t month, uint32_t day);
+
+/**
+ * Translates an absolute day number to a 
+ * Julian day number.
+ *
+ * @param The absolute day number
+ */
+int64_t absolute_day_number_to_julian_day_number(int64_t absolute_day);
+
+/**
+ * Translates a Julian day number to an 
+ * absolute day number.  
+ *
+ * @param The Julian day number
+ */
+int64_t julian_day_number_to_absolute_day_number(int64_t julian_day);
+
+/**
+ * Given a supplied Julian Day Number, populates a year, month, and day
+ * with the date in the Gregorian Proleptic calendar which corresponds to
+ * the given Julian Day Number.
+ *
+ * @param Julian Day Number
+ * @param Pointer to year to populate
+ * @param Pointer to month to populate
+ * @param Pointer to the day to populate
+ */
+void gregorian_date_from_julian_day_number(int64_t julian_day
+                                         , uint32_t *year_out
+                                         , uint32_t *month_out
+                                         , uint32_t *day_out);
+
+/**
+ * Given a supplied Absolute Day Number, populates a year, month, and day
+ * with the date in the Gregorian Proleptic calendar which corresponds to
+ * the given Absolute Day Number.
+ *
+ * @param Absolute Day Number
+ * @param Pointer to year to populate
+ * @param Pointer to month to populate
+ * @param Pointer to the day to populate
+ */
+void gregorian_date_from_absolute_day_number(int64_t absolute_day
+                                           , uint32_t *year_out
+                                           , uint32_t *month_out
+                                           , uint32_t *day_out);
+
+/**
+ * Returns the number of days in a particular year.
+ *
+ * @param year to evaluate
+ * @param calendar to use
+ */
+uint32_t days_in_year(uint32_t year, enum calendar calendar);
+
+/**
+ * Returns the number of days in a particular Gregorian Proleptic calendar year.
+ *
+ * @param year to evaluate
+ */
+uint32_t days_in_year_gregorian(uint32_t year);
+
+/**
+ * Returns the number of days in a particular Julian Proleptic calendar year.
+ *
+ * @param year to evaluate
+ */
+uint32_t days_in_year_julian(uint32_t year);
+
+#define NUM_LEAP_YEARS(y, c) ((c) == GREGORIAN \
+    ? number_of_leap_years_gregorian((y)) \
+    : number_of_leap_years_julian((y)))
+
+/**
+ * Returns the number of leap years that have
+ * occurred in the Julian Proleptic calendar
+ * up to the supplied year.
+ *
+ * @param year to evaluate (1 - 9999)
+ */
+int32_t number_of_leap_years_julian(uint32_t year);
+
+/**
+ * Returns the number of leap years that have
+ * occurred in the Gregorian Proleptic calendar
+ * up to the supplied year.
+ *
+ * @param year to evaluate (1 - 9999)
+ */
+int32_t number_of_leap_years_gregorian(uint32_t year);
+
+/**
+ * Returns the number of days in a month, given
+ * a year and a month in the Gregorian calendar.
+ *
+ * @param Year in Gregorian Proleptic calendar
+ * @param Month in date
+ */
+uint32_t days_in_gregorian_year_month(uint32_t year, uint32_t month);
+
+/**
+ * Returns the number of the day in a week.
+ *
+ * @see temporal_to_number_days()
+ *
+ * Return values:
+ *
+ * Day            Day Number  Sunday first day?
+ * -------------- ----------- -----------------
+ * Sunday         0           true
+ * Monday         1           true
+ * Tuesday        2           true
+ * Wednesday      3           true
+ * Thursday       4           true
+ * Friday         5           true
+ * Saturday       6           true
+ * Sunday         6           false
+ * Monday         0           false
+ * Tuesday        1           false
+ * Wednesday      2           false
+ * Thursday       3           false
+ * Friday         4           false
+ * Saturday       5           false
+ *
+ * @param Number of days since start of Gregorian calendar.
+ * @param Consider Sunday the first day of the week?
+ */
+uint32_t day_of_week(int64_t day_number, bool sunday_is_first_day_of_week);
+
+/**
+ * Given a year, month, and day, returns whether the date is 
+ * valid for the Gregorian proleptic calendar.
+ *
+ * @param The year
+ * @param The month
+ * @param The day
+ */
+bool is_valid_gregorian_date(uint32_t year, uint32_t month, uint32_t day);
+
+/**
+ * Returns whether the supplied date components are within the 
+ * range of the UNIX epoch.
+ *
+ * Times in the range of 1970-01-01T00:00:00 to 2038-01-19T03:14:07
+ *
+ * @param Year
+ * @param Month
+ * @param Day
+ * @param Hour
+ * @param Minute
+ * @param Second
+ */
+bool in_unix_epoch_range(uint32_t year
+                       , uint32_t month
+                       , uint32_t day
+                       , uint32_t hour
+                       , uint32_t minute
+                       , uint32_t second);
+
+/* 
+ * I don't like using these defines, but probably good to keep in sync
+ * with MySQL's week mode stuff.. 
+ */
+#define DRIZZLE_WEEK_MODE_MONDAY_FIRST_DAY   1
+#define DRIZZLE_WEEK_MODE_USE_ISO_8601_1988  2
+#define DRIZZLE_WEEK_MODE_WEEK_RANGE_IS_ORDINAL 4
+
+/**
+ * Returns the number of the week from a supplied year, month, and
+ * date in the Gregorian proleptic calendar.
+ *
+ * The week number returned will depend on the values of the
+ * various boolean flags passed to the function.
+ *
+ * The flags influence returned values in the following ways:
+ *
+ * sunday_is_first_day_of_week
+ *
+ * If TRUE, Sunday is first day of week
+ * If FALSE,	Monday is first day of week
+ *
+ * week_range_is_ordinal
+ *
+ * If FALSE, the week is in range 0-53
+ *
+ * Week 0 is returned for the the last week of the previous year (for
+ * a date at start of january) In this case one can get 53 for the
+ * first week of next year.  This flag ensures that the week is
+ * relevant for the given year. 
+ *
+ * If TRUE, the week is in range 1-53.
+ *
+ * In this case one may get week 53 for a date in January (when
+ * the week is that last week of previous year) and week 1 for a
+ * date in December.
+ *
+ * use_iso_8601_1988
+ *
+ * If TRUE, the weeks are numbered according to ISO 8601:1988
+ *
+ * ISO 8601:1988 means that if the week containing January 1 has
+ * four or more days in the new year, then it is week 1;
+ * Otherwise it is the last week of the previous year, and the
+ * next week is week 1.
+ *
+ * If FALSE, the week that contains the first 'first-day-of-week' is week 1.
+ *
+ * @param Subject year
+ * @param Subject month
+ * @param Subject day
+ * @param Is sunday the first day of the week?
+ * @param Is the week range ordinal?
+ * @param Should we use ISO 8601:1988 rules?
+ * @param Pointer to a uint32_t to hold the resulting year, which 
+ *        may be incremented or decremented depending on flags
+ */
+uint32_t week_number_from_gregorian_date(uint32_t year
+                                       , uint32_t month
+                                       , uint32_t day
+                                       , bool sunday_is_first_day_of_week
+                                       , bool week_range_is_ordinal
+                                       , bool use_iso_8601_1988
+                                       , uint32_t *year_out);
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* DRIZZLED_CALENDAR_H */

=== modified file 'drizzled/drizzled.cc'
--- drizzled/drizzled.cc	2009-01-28 01:17:32 +0000
+++ drizzled/drizzled.cc	2009-01-28 02:39:29 +0000
@@ -45,7 +45,7 @@
 #include <drizzled/errmsg.h>
 #include <drizzled/unireg.h>
 #include <drizzled/plugin_scheduling.h>
-
+#include "drizzled/temporal_format.h" /* For init_temporal_formats() */
 
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -1895,6 +1895,9 @@
     return 1;
   if (set_var_init())
     return 1;
+  /* Creates static regex matching for temporal values */
+  if (! init_temporal_formats())
+    return 1;
   /*
     Process a comma-separated character set list and choose
     the first available character set. This is mostly for

=== modified file 'drizzled/error.cc'
--- drizzled/error.cc	2008-12-22 02:20:48 +0000
+++ drizzled/error.cc	2009-01-28 02:39:29 +0000
@@ -1393,7 +1393,25 @@
 /* ER_UNSUPPORTED_ACTION_ON_VIRTUAL_COLUMN */
 N_("'%s' is not yet supported for virtual columns."),
 /* ER_CONST_EXPR_IN_VCOL */
-N_("Constant expression in virtual column function is not allowed.")
+N_("Constant expression in virtual column function is not allowed."),
+/* ER_UNKNOWN_TEMPORAL_TYPE */
+N_("Encountered an unknown temporal type."),
+/* ER_INVALID_STRING_FORMAT_FOR_DATE */
+N_("Received an invalid string format '%s' for a date value."),
+/* ER_INVALID_STRING_FORMAT_FOR_TIME */
+N_("Received an invalid string format '%s' for a time value."),
+/* ER_INVALID_UNIX_TIMESTAMP_VALUE */
+N_("Received an invalid value '%s' for a UNIX timestamp."),
+/* ER_INVALID_DATETIME_VALUE */
+N_("Received an invalid datetime value '%s'."),
+/* ER_INVALID_NULL_ARGUMENT */
+N_("Received a NULL argument for function '%s'."),
+/* ER_INVALID_NEGATIVE_ARGUMENT */
+N_("Received an invalid negative argument '%s' for function '%s'."),
+/* ER_ARGUMENT_OUT_OF_RANGE */
+N_("Received an out-of-range argument '%s' for function '%s'."),
+/* ER_INVALID_TIME_VALUE */
+N_("Received an invalid time value '%s'."),
 };
 
 const char * error_message(unsigned int code)

=== modified file 'drizzled/error.h'
--- drizzled/error.h	2008-12-10 07:43:50 +0000
+++ drizzled/error.h	2009-01-28 02:39:29 +0000
@@ -711,7 +711,16 @@
   ER_WARNING_NON_DEFAULT_VALUE_FOR_VIRTUAL_COLUMN,
   ER_UNSUPPORTED_ACTION_ON_VIRTUAL_COLUMN,
   ER_CONST_EXPR_IN_VCOL,
-  ER_ERROR_LAST= ER_CONST_EXPR_IN_VCOL
+  ER_UNKNOWN_TEMPORAL_TYPE,
+  ER_INVALID_STRING_FORMAT_FOR_DATE,
+  ER_INVALID_STRING_FORMAT_FOR_TIME,
+  ER_INVALID_UNIX_TIMESTAMP_VALUE,
+  ER_INVALID_DATETIME_VALUE,
+  ER_INVALID_NULL_ARGUMENT,
+  ER_INVALID_NEGATIVE_ARGUMENT,
+  ER_ARGUMENT_OUT_OF_RANGE,
+  ER_INVALID_TIME_VALUE,
+  ER_ERROR_LAST= ER_ARGUMENT_OUT_OF_RANGE
 };
 
 #ifdef  __cplusplus

=== modified file 'drizzled/function/time/year.cc'
--- drizzled/function/time/year.cc	2008-12-12 20:40:37 +0000
+++ drizzled/function/time/year.cc	2009-01-28 02:39:29 +0000
@@ -17,16 +17,81 @@
  *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <drizzled/server_includes.h>
+#include "drizzled/server_includes.h"
+#include "drizzled/temporal.h"
 #include CSTDINT_H
-#include <drizzled/function/time/year.h>
+#include "drizzled/function/time/year.h"
+#include "drizzled/error.h"
 
 int64_t Item_func_year::val_int()
 {
-  assert(fixed == 1);
-  DRIZZLE_TIME ltime;
-  (void) get_arg0_date(&ltime, TIME_FUZZY_DATE);
-  return (int64_t) ltime.year;
+  assert(fixed);
+
+  if (args[0]->is_null())
+  {
+    /* For NULL argument, we return a NULL result */
+    null_value= true;
+    return 0;
+  }
+
+  /* Grab the first argument as a DateTime object */
+  drizzled::DateTime temporal;
+  Item_result arg0_result_type= args[0]->result_type();
+  
+  switch (arg0_result_type)
+  {
+    case STRING_RESULT:
+      {
+        char buff[40];
+        String tmp(buff,sizeof(buff), &my_charset_utf8_bin);
+        String *res= args[0]->val_str(&tmp);
+        if (! temporal.from_string(res->c_ptr(), res->length()))
+        {
+          /* 
+          * Could not interpret the function argument as a temporal value, 
+          * so throw an error and return 0
+          */
+          my_error(ER_INVALID_DATETIME_VALUE, MYF(0), res->c_ptr());
+          return 0;
+        }
+      }
+      break;
+    case INT_RESULT:
+      if (! temporal.from_int64_t(args[0]->val_int()))
+      {
+        /* 
+        * Could not interpret the function argument as a temporal value, 
+        * so throw an error and return 0
+        */
+        null_value= true;
+        char buff[40];
+        String tmp(buff,sizeof(buff), &my_charset_utf8_bin);
+        String *res;
+
+        res= args[0]->val_str(&tmp);
+
+        my_error(ER_INVALID_DATETIME_VALUE, MYF(0), res->c_ptr());
+        return 0;
+      }
+      break;
+    default:
+      {
+        /* 
+        * Could not interpret the function argument as a temporal value, 
+        * so throw an error and return 0
+        */
+        null_value= true;
+        char buff[40];
+        String tmp(buff,sizeof(buff), &my_charset_utf8_bin);
+        String *res;
+
+        res= args[0]->val_str(&tmp);
+
+        my_error(ER_INVALID_DATETIME_VALUE, MYF(0), res->c_ptr());
+        return 0;
+      }
+  }
+  return (int64_t) temporal.years();
 }
 
 /* information about this Item tree monotonicity
@@ -54,13 +119,22 @@
 int64_t Item_func_year::val_int_endpoint(bool left_endp, bool *incl_endp)
 {
   assert(fixed == 1);
-  DRIZZLE_TIME ltime;
-  if (get_arg0_date(&ltime, TIME_FUZZY_DATE))
+
+  if (args[0]->is_null())
   {
     /* got NULL, leave the incl_endp intact */
     return INT64_MIN;
   }
 
+  /* Grab the first argument as a DateTime object */
+  drizzled::DateTime temporal;
+  
+  if (! temporal.from_int64_t(args[0]->val_int()))
+  {
+    /* got bad DateTime, leave the incl_endp intact */
+    return INT64_MIN;
+  }
+
   /*
     Handle the special but practically useful case of datetime values that
     point to year bound ("strictly less" comparison stays intact) :
@@ -72,11 +146,11 @@
 
       col < '2007-09-15 23:00:00'  -> YEAR(col) <= 2007
   */
-  if (!left_endp && ltime.day == 1 && ltime.month == 1 &&
-      !(ltime.hour || ltime.minute || ltime.second || ltime.second_part))
+  if (!left_endp && temporal.days() == 1 && temporal.months() == 1 &&
+      ! (temporal.hours() || temporal.minutes() || temporal.seconds() || temporal.useconds()))
     ; /* do nothing */
   else
     *incl_endp= true;
-  return ltime.year;
+  return (int64_t) temporal.years();
 }
 

=== added file 'drizzled/temporal.cc'
--- drizzled/temporal.cc	1970-01-01 00:00:00 +0000
+++ drizzled/temporal.cc	2009-01-28 02:39:29 +0000
@@ -0,0 +1,1191 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  Authors:
+ *
+ *  Jay Pipes <jay.pipes@sun.com>
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *
+ * Implementation of the server's temporal class API
+ *
+ * @todo
+ *
+ * Move to completed ValueObject API, which would remove the from_xxx() methods
+ * and replace them with constructors which take other ValueObject instances as
+ * their single parameter.
+ */
+
+#include <vector>
+#include <string.h>
+
+#include "drizzled/global.h"
+#include "mystrings/m_ctype.h"
+#include "drizzled/my_decimal.h"
+#include "drizzled/temporal.h"
+#include "drizzled/temporal_format.h"
+
+extern std::vector<drizzled::TemporalFormat *> known_datetime_formats;
+extern std::vector<drizzled::TemporalFormat *> known_date_formats;
+extern std::vector<drizzled::TemporalFormat *> known_time_formats;
+
+namespace drizzled 
+{
+
+Temporal::Temporal()
+:
+  _calendar(GREGORIAN)
+, _years(0)
+, _months(0)
+, _days(0)
+, _hours(0)
+, _minutes(0)
+, _seconds(0)
+, _epoch_seconds(0)
+, _useconds(0)
+, _nseconds(0)
+{}
+
+uint64_t Temporal::_cumulative_seconds_in_time() const
+{
+  return (uint64_t) ((_hours * INT64_C(3600)) 
+      + (_minutes * INT64_C(60)) 
+      + _seconds);
+}
+
+void Temporal::set_epoch_seconds()
+{
+  /* 
+   * If the temporal is in the range of a timestamp, set 
+   * the epoch_seconds member variable
+   */
+  if (in_unix_epoch_range(_years, _months, _days, _hours, _minutes, _seconds))
+  {
+    time_t result_time;
+    struct tm broken_time;
+
+    broken_time.tm_sec= _seconds;
+    broken_time.tm_min= _minutes;
+    broken_time.tm_hour= _hours;
+    broken_time.tm_mday= _days; /* Drizzle format uses ordinal, standard tm does too! */
+    broken_time.tm_mon= _months - 1; /* Drizzle format uses ordinal, standard tm does NOT! */
+    broken_time.tm_year= _years - 1900; /* tm_year expects range of 70 - 38 */
+
+    result_time= timegm(&broken_time);
+
+    _epoch_seconds= result_time;
+  }
+}
+
+bool Date::from_string(const char *from, size_t from_len)
+{
+  /* 
+   * Loop through the known date formats and see if 
+   * there is a match.
+   */
+  bool matched= false;
+  TemporalFormat *current_format;
+  std::vector<TemporalFormat *>::iterator current= known_date_formats.begin();
+
+  while (current != known_date_formats.end())
+  {
+    current_format= *current;
+    if (current_format->matches(from, from_len, this))
+    {
+      matched= true;
+      break;
+    }
+    current++;
+  }
+
+  if (! matched)
+    return false;
+
+  set_epoch_seconds();
+  return is_valid();
+}
+
+bool DateTime::from_string(const char *from, size_t from_len)
+{
+  /* 
+   * Loop through the known datetime formats and see if 
+   * there is a match.
+   */
+  bool matched= false;
+  TemporalFormat *current_format;
+  std::vector<TemporalFormat *>::iterator current= known_datetime_formats.begin();
+
+  while (current != known_datetime_formats.end())
+  {
+    current_format= *current;
+    if (current_format->matches(from, from_len, this))
+    {
+      matched= true;
+      break;
+    }
+    current++;
+  }
+
+  if (! matched)
+    return false;
+
+  set_epoch_seconds();
+  return is_valid();
+}
+
+/*
+ * Comparison operators for Time against another Time
+ * are easy.  We simply compare the cumulative time
+ * value of each.
+ */
+bool Time::operator==(const Time& rhs)
+{
+  return (
+          _hours == rhs._hours
+       && _minutes == rhs._minutes
+       && _seconds == rhs._seconds
+       && _useconds == rhs._useconds
+       && _nseconds == rhs._nseconds
+      );
+}
+bool Time::operator!=(const Time& rhs)
+{
+  return ! (*this == rhs);
+}
+bool Time::operator<(const Time& rhs)
+{
+  return (_cumulative_seconds_in_time() < rhs._cumulative_seconds_in_time());
+}
+bool Time::operator<=(const Time& rhs)
+{
+  return (_cumulative_seconds_in_time() <= rhs._cumulative_seconds_in_time());
+}
+bool Time::operator>(const Time& rhs)
+{
+  return (_cumulative_seconds_in_time() > rhs._cumulative_seconds_in_time());
+}
+bool Time::operator>=(const Time& rhs)
+{
+  return (_cumulative_seconds_in_time() >= rhs._cumulative_seconds_in_time());
+}
+
+/** 
+ * Subtracting one Time value from another can yield
+ * a new Time instance.
+ *
+ * This operator is called in the following situation:
+ *
+ * @code
+ * drizzled::Time lhs;
+ * lhs.from_string("20:00:00");
+ * drizzled::Time rhs;
+ * rhs.from_string("19:00:00");
+ *
+ * drizzled::Time result= lhs - rhs;
+ * @endcode
+ *
+ * @note
+ *
+ * Subtracting a larger time value from a smaller one
+ * should throw an exception at some point.  The result
+ * of such an operator should be a TemporalInterval, not
+ * a Time instance, since a negative time is not possible.
+ */
+const Time Time::operator-(const Time& rhs)
+{
+  Time result;
+
+  int64_t second_diff= _cumulative_seconds_in_time() - rhs._cumulative_seconds_in_time();
+  result._hours= second_diff % DRIZZLE_SECONDS_IN_HOUR;
+  second_diff-= (result._hours * DRIZZLE_SECONDS_IN_HOUR);
+  result._minutes= second_diff % DRIZZLE_SECONDS_IN_MINUTE;
+  second_diff-= (result._minutes * DRIZZLE_SECONDS_IN_MINUTE);
+  result._seconds= second_diff;
+  
+  return result;
+}
+const Time Time::operator+(const Time& rhs)
+{
+  Time result;
+  int64_t second_diff= _cumulative_seconds_in_time() + rhs._cumulative_seconds_in_time();
+  result._hours= second_diff % DRIZZLE_SECONDS_IN_HOUR;
+  second_diff-= (result._hours * DRIZZLE_SECONDS_IN_HOUR);
+  result._minutes= second_diff % DRIZZLE_SECONDS_IN_MINUTE;
+  second_diff-= (result._minutes * DRIZZLE_SECONDS_IN_MINUTE);
+  result._seconds= second_diff;
+  /** 
+   * @TODO Once exceptions are supported, we should raise an error here if
+   *       the result Time is not valid?
+   */
+  return result;
+}
+
+/*
+ * Variation of + and - operator which returns a reference to the left-hand
+ * side Time object and adds the right-hand side to itself.
+ */
+Time& Time::operator+=(const Time& rhs)
+{
+  int64_t second_diff= _cumulative_seconds_in_time() + rhs._cumulative_seconds_in_time();
+  _hours= second_diff % DRIZZLE_SECONDS_IN_HOUR;
+  second_diff-= (_hours * DRIZZLE_SECONDS_IN_HOUR);
+  _minutes= second_diff % DRIZZLE_SECONDS_IN_MINUTE;
+  second_diff-= (_minutes * DRIZZLE_SECONDS_IN_MINUTE);
+  _seconds= second_diff;
+  /** 
+   * @TODO Once exceptions are supported, we should raise an error here if
+   *       the result Time is not valid?
+   */
+  return *this;
+}
+Time& Time::operator-=(const Time& rhs)
+{
+  int64_t second_diff= _cumulative_seconds_in_time() - rhs._cumulative_seconds_in_time();
+  _hours= second_diff % DRIZZLE_SECONDS_IN_HOUR;
+  second_diff-= (_hours * DRIZZLE_SECONDS_IN_HOUR);
+  _minutes= second_diff % DRIZZLE_SECONDS_IN_MINUTE;
+  second_diff-= (_minutes * DRIZZLE_SECONDS_IN_MINUTE);
+  _seconds= second_diff;
+  /** 
+   * @TODO Once exceptions are supported, we should raise an error here if
+   *       the result Time is not valid?
+   */
+  return *this;
+}
+
+/*
+ * Comparison operators for Date against another Date
+ * are easy.  We simply compare the cumulative
+ * value of each.
+ */
+bool Date::operator==(const Date& rhs)
+{
+  return (
+          _years == rhs._years
+       && _months == rhs._months
+       && _days == rhs._days
+      );
+}
+bool Date::operator!=(const Date& rhs)
+{
+  return ! (*this == rhs);
+}
+bool Date::operator<(const Date& rhs)
+{
+  int days_left= julian_day_number_from_gregorian_date(_years, _months, _days);
+  int days_right= julian_day_number_from_gregorian_date(rhs._years, rhs._months, rhs._days);
+  return (days_left < days_right);
+}
+bool Date::operator<=(const Date& rhs)
+{
+  int days_left= julian_day_number_from_gregorian_date(_years, _months, _days);
+  int days_right= julian_day_number_from_gregorian_date(rhs._years, rhs._months, rhs._days);
+  return (days_left <= days_right);
+}
+bool Date::operator>(const Date& rhs)
+{
+  return ! (*this <= rhs);
+}
+bool Date::operator>=(const Date& rhs)
+{
+  return ! (*this < rhs);
+}
+
+/*
+ * Comparison operators for DateTime against another DateTime
+ * are easy.  We simply compare the cumulative time
+ * value of each.
+ */
+bool DateTime::operator==(const DateTime& rhs)
+{
+  return (
+          _years == rhs._years
+       && _months == rhs._months
+       && _days == rhs._days
+       && _hours == rhs._hours
+       && _minutes == rhs._minutes
+       && _seconds == rhs._seconds
+       && _useconds == rhs._useconds
+       && _nseconds == rhs._nseconds
+      );
+}
+bool DateTime::operator!=(const DateTime& rhs)
+{
+  return ! (*this == rhs);
+}
+bool DateTime::operator<(const DateTime& rhs)
+{
+  int days_left= julian_day_number_from_gregorian_date(_years, _months, _days);
+  int days_right= julian_day_number_from_gregorian_date(rhs._years, rhs._months, rhs._days);
+  if (days_left < days_right)
+    return true;
+  else if (days_left > days_right)
+    return false;
+  /* Here if both dates are the same, so compare times */
+  return (_cumulative_seconds_in_time() < rhs._cumulative_seconds_in_time());
+}
+bool DateTime::operator<=(const DateTime& rhs)
+{
+  int days_left= julian_day_number_from_gregorian_date(_years, _months, _days);
+  int days_right= julian_day_number_from_gregorian_date(rhs._years, rhs._months, rhs._days);
+  if (days_left < days_right)
+    return true;
+  else if (days_left > days_right)
+    return false;
+  /* Here if both dates are the same, so compare times */
+  return (_cumulative_seconds_in_time() <= rhs._cumulative_seconds_in_time());
+}
+bool DateTime::operator>(const DateTime& rhs)
+{
+  return ! (*this <= rhs);
+}
+bool DateTime::operator>=(const DateTime& rhs)
+{
+  return ! (*this < rhs);
+}
+
+/** 
+ * We can add or subtract a Time value to/from a DateTime value 
+ * as well...it always produces a DateTime.
+ */
+const DateTime DateTime::operator-(const Time& rhs)
+{
+  DateTime result;
+
+  /* 
+   * First, we set the resulting DATE pieces equal to our 
+   * left-hand side DateTime's DATE components. Then, deal with 
+   * the time components.
+   */
+  result._years= _years;
+  result._months= _months;
+  result._days= _days;
+
+  int64_t second_diff= _cumulative_seconds_in_time() - rhs._cumulative_seconds_in_time();
+
+  /* 
+   * The resulting diff might be negative.  If it is, that means that 
+   * we have subtracting a larger time piece from the datetime, like so:
+   *
+   * x = DateTime("2007-06-09 09:30:00") - Time("16:30:00");
+   *
+   * In these cases, we need to subtract a day from the resulting
+   * DateTime.
+   */
+  if (second_diff < 0)
+    result._days--;
+
+  result._hours= second_diff % DRIZZLE_SECONDS_IN_HOUR;
+  second_diff-= (result._hours * DRIZZLE_SECONDS_IN_HOUR);
+  result._minutes= second_diff % DRIZZLE_SECONDS_IN_MINUTE;
+  second_diff-= (result._minutes * DRIZZLE_SECONDS_IN_MINUTE);
+  result._seconds= second_diff;
+
+  /* Handle the microsecond precision */
+  int64_t microsecond_diff= _useconds - rhs._useconds;
+  if (microsecond_diff < 0)
+  {
+    microsecond_diff= (-1 * microsecond_diff);
+    result._seconds--;
+  }
+  result._useconds= microsecond_diff;
+
+  return result;
+}
+const DateTime DateTime::operator+(const Time& rhs)
+{
+  DateTime result;
+
+  /* 
+   * First, we set the resulting DATE pieces equal to our 
+   * left-hand side DateTime's DATE components. Then, deal with 
+   * the time components.
+   */
+  result._years= _years;
+  result._months= _months;
+  result._days= _days;
+
+  int64_t second_diff= _cumulative_seconds_in_time() + rhs._cumulative_seconds_in_time();
+
+  /* 
+   * The resulting seconds might be more than a day.  If do, 
+   * adjust our resulting days up 1.
+   */
+  if (second_diff >= DRIZZLE_SECONDS_IN_DAY)
+  {
+    result._days++;
+    second_diff-= (result._days * DRIZZLE_SECONDS_IN_DAY);
+  }
+
+  result._hours= second_diff % DRIZZLE_SECONDS_IN_HOUR;
+  second_diff-= (result._hours * DRIZZLE_SECONDS_IN_HOUR);
+  result._minutes= second_diff % DRIZZLE_SECONDS_IN_MINUTE;
+  second_diff-= (result._minutes * DRIZZLE_SECONDS_IN_MINUTE);
+  result._seconds= second_diff;
+
+  /* Handle the microsecond precision */
+  int64_t microsecond_diff= _useconds - rhs._useconds;
+  if (microsecond_diff < 0)
+  {
+    microsecond_diff= (-1 * microsecond_diff);
+    result._seconds--;
+  }
+  result._useconds= microsecond_diff;
+
+  return result;
+}
+
+/*
+ * Variation of + and - operator which returns a reference to the left-hand
+ * side DateTime object and adds the right-hand side Time to itself.
+ */
+DateTime& DateTime::operator+=(const Time& rhs)
+{
+  int64_t second_diff= _cumulative_seconds_in_time() + rhs._cumulative_seconds_in_time();
+  /* 
+   * The resulting seconds might be more than a day.  If do, 
+   * adjust our resulting days up 1.
+   */
+  if (second_diff >= DRIZZLE_SECONDS_IN_DAY)
+  {
+    _days++;
+    second_diff-= (_days * DRIZZLE_SECONDS_IN_DAY);
+  }
+
+  _hours= second_diff % DRIZZLE_SECONDS_IN_HOUR;
+  second_diff-= (_hours * DRIZZLE_SECONDS_IN_HOUR);
+  _minutes= second_diff % DRIZZLE_SECONDS_IN_MINUTE;
+  second_diff-= (_minutes * DRIZZLE_SECONDS_IN_MINUTE);
+  _seconds= second_diff;
+
+  /* Handle the microsecond precision */
+  int64_t microsecond_diff= _useconds - rhs._useconds;
+  if (microsecond_diff < 0)
+  {
+    microsecond_diff= (-1 * microsecond_diff);
+    _seconds--;
+  }
+  _useconds= microsecond_diff;
+  /** 
+   * @TODO Once exceptions are supported, we should raise an error here if
+   *       the result Time is not valid?
+   */
+  return *this;
+}
+DateTime& DateTime::operator-=(const Time& rhs)
+{
+  int64_t second_diff= _cumulative_seconds_in_time() - rhs._cumulative_seconds_in_time();
+
+  /* 
+   * The resulting diff might be negative.  If it is, that means that 
+   * we have subtracting a larger time piece from the datetime, like so:
+   *
+   * x = DateTime("2007-06-09 09:30:00");
+   * x-= Time("16:30:00");
+   *
+   * In these cases, we need to subtract a day from the resulting
+   * DateTime.
+   */
+  if (second_diff < 0)
+    _days--;
+
+  _hours= second_diff % DRIZZLE_SECONDS_IN_HOUR;
+  second_diff-= (_hours * DRIZZLE_SECONDS_IN_HOUR);
+  _minutes= second_diff % DRIZZLE_SECONDS_IN_MINUTE;
+  second_diff-= (_minutes * DRIZZLE_SECONDS_IN_MINUTE);
+  _seconds= second_diff;
+
+  /* Handle the microsecond precision */
+  int64_t microsecond_diff= _useconds - rhs._useconds;
+  if (microsecond_diff < 0)
+  {
+    microsecond_diff= (-1 * microsecond_diff);
+    _seconds--;
+  }
+  _useconds= microsecond_diff;
+  /** 
+   * @TODO Once exceptions are supported, we should raise an error here if
+   *       the result Time is not valid?
+   */
+  return *this;
+}
+
+/**
+ * We can add/subtract two Dates to/from each other.  The result
+ * is always another Date instance.
+ */
+const Date Date::operator-(const Date &rhs)
+{
+  /* Figure out the difference in days between the two dates */
+  int64_t day_left= julian_day_number_from_gregorian_date(_years, _months, _days);
+  int64_t day_right= julian_day_number_from_gregorian_date(rhs._years, rhs._months, rhs._days);
+  int64_t day_diff= day_left - day_right;
+
+  Date result;
+  /* Now re-compose the Date's structure from the resulting Julian Day Number */
+  gregorian_date_from_julian_day_number(day_diff, &result._years, &result._months, &result._days);
+  return result;
+}
+const Date Date::operator+(const Date &rhs)
+{
+  /* 
+   * Figure out the new Julian Day Number by adding the JDNs of both
+   * dates together.
+   */
+  int64_t day_left= julian_day_number_from_gregorian_date(_years, _months, _days);
+  int64_t day_right= julian_day_number_from_gregorian_date(rhs._years, rhs._months, rhs._days);
+  int64_t day_diff= day_left + day_right;
+
+  /** @TODO Need an exception check here for bounds of JDN... */
+
+  Date result;
+  /* Now re-compose the Date's structure from the resulting Julian Day Number */
+  gregorian_date_from_julian_day_number(day_diff, &result._years, &result._months, &result._days);
+  return result;
+}
+/* Similar to the above, but we add/subtract the right side to this object itself */
+Date& Date::operator-=(const Date &rhs)
+{
+  int64_t day_left= julian_day_number_from_gregorian_date(_years, _months, _days);
+  int64_t day_right= julian_day_number_from_gregorian_date(rhs._years, rhs._months, rhs._days);
+  int64_t day_diff= day_left - day_right;
+
+  /* Now re-compose the Date's structure from the resulting Julian Day Number */
+  gregorian_date_from_julian_day_number(day_diff, &_years, &_months, &_days);
+  return *this;
+}
+Date& Date::operator+=(const Date &rhs)
+{
+  /* 
+   * Figure out the new Julian Day Number by adding the JDNs of both
+   * dates together.
+   */
+  int64_t day_left= julian_day_number_from_gregorian_date(_years, _months, _days);
+  int64_t day_right= julian_day_number_from_gregorian_date(rhs._years, rhs._months, rhs._days);
+  int64_t day_diff= day_left + day_right;
+
+  /** @TODO Need an exception check here for bounds of JDN... */
+
+  /* Now re-compose the Date's structure from the resulting Julian Day Number */
+  gregorian_date_from_julian_day_number(day_diff, &_years, &_months, &_days);
+  return *this;
+}
+
+/**
+ * We can add/subtract two DateTimes to/from each other.  The result
+ * is always another DateTime instance.
+ */
+const DateTime DateTime::operator-(const DateTime &rhs)
+{
+  /* Figure out the difference in days between the two dates. */
+  int64_t day_left= julian_day_number_from_gregorian_date(_years, _months, _days);
+  int64_t day_right= julian_day_number_from_gregorian_date(rhs._years, rhs._months, rhs._days);
+  int64_t day_diff= day_left - day_right;
+
+  DateTime result;
+  /* Now re-compose the Date's structure from the resulting Julian Day Number */
+  gregorian_date_from_julian_day_number(day_diff, &result._years, &result._months, &result._days);
+
+  /* And now handle the time components */
+  int64_t second_diff= _cumulative_seconds_in_time() - rhs._cumulative_seconds_in_time();
+
+  /* 
+   * The resulting diff might be negative.  If it is, that means that 
+   * we have subtracting a larger time piece from the datetime, like so:
+   *
+   * x = DateTime("2007-06-09 09:30:00");
+   * x-= Time("16:30:00");
+   *
+   * In these cases, we need to subtract a day from the resulting
+   * DateTime.
+   */
+  if (second_diff < 0)
+    _days--;
+
+  result._hours= second_diff % DRIZZLE_SECONDS_IN_HOUR;
+  second_diff-= (result._hours * DRIZZLE_SECONDS_IN_HOUR);
+  result._minutes= second_diff % DRIZZLE_SECONDS_IN_MINUTE;
+  second_diff-= (result._minutes * DRIZZLE_SECONDS_IN_MINUTE);
+  result._seconds= second_diff;
+
+  /* Handle the microsecond precision */
+  int64_t microsecond_diff= _useconds - rhs._useconds;
+  if (microsecond_diff < 0)
+  {
+    microsecond_diff= (-1 * microsecond_diff);
+    result._seconds--;
+  }
+  result._useconds= microsecond_diff;
+
+  return result;
+}
+const DateTime DateTime::operator+(const DateTime &rhs)
+{
+  /* 
+   * Figure out the new Julian Day Number by adding the JDNs of both
+   * dates together.
+   */
+  int64_t day_left= julian_day_number_from_gregorian_date(_years, _months, _days);
+  int64_t day_right= julian_day_number_from_gregorian_date(rhs._years, rhs._months, rhs._days);
+  int64_t day_diff= day_left + day_right;
+
+  /** @TODO Need an exception check here for bounds of JDN... */
+
+  DateTime result;
+  /* Now re-compose the Date's structure from the resulting Julian Day Number */
+  gregorian_date_from_julian_day_number(day_diff, &result._years, &result._months, &result._days);
+
+  /* And now handle the time components */
+  int64_t second_diff= _cumulative_seconds_in_time() + rhs._cumulative_seconds_in_time();
+
+  /* 
+   * The resulting seconds might be more than a day.  If do, 
+   * adjust our resulting days up 1.
+   */
+  if (second_diff >= DRIZZLE_SECONDS_IN_DAY)
+  {
+    result._days++;
+    second_diff-= (result._days * DRIZZLE_SECONDS_IN_DAY);
+  }
+
+  result._hours= second_diff % DRIZZLE_SECONDS_IN_HOUR;
+  second_diff-= (result._hours * DRIZZLE_SECONDS_IN_HOUR);
+  result._minutes= second_diff % DRIZZLE_SECONDS_IN_MINUTE;
+  second_diff-= (result._minutes * DRIZZLE_SECONDS_IN_MINUTE);
+  result._seconds= second_diff;
+
+  /* Handle the microsecond precision */
+  int64_t microsecond_diff= _useconds - rhs._useconds;
+  if (microsecond_diff < 0)
+  {
+    microsecond_diff= (-1 * microsecond_diff);
+    result._seconds--;
+  }
+  result._useconds= microsecond_diff;
+
+  return result;
+}
+/* Similar to the above, but we add/subtract the right side to this object itself */
+DateTime& DateTime::operator-=(const DateTime &rhs)
+{
+  /* Figure out the difference in days between the two dates.  */
+  int64_t day_left= julian_day_number_from_gregorian_date(_years, _months, _days);
+  int64_t day_right= julian_day_number_from_gregorian_date(rhs._years, rhs._months, rhs._days);
+  int64_t day_diff= day_left - day_right;
+
+  /* Now re-compose the Date's structure from the ng Julian Day Number */
+  gregorian_date_from_julian_day_number(day_diff, &_years, &_months, &_days);
+
+  /* And now handle the time components */
+  int64_t second_diff= _cumulative_seconds_in_time() - rhs._cumulative_seconds_in_time();
+
+  /* 
+   * The resulting diff might be negative.  If it is, that means that 
+   * we have subtracting a larger time piece from the datetime, like so:
+   *
+   * x = DateTime("2007-06-09 09:30:00");
+   * x-= Time("16:30:00");
+   *
+   * In these cases, we need to subtract a day from the ng
+   * DateTime.
+   */
+  if (second_diff < 0)
+    _days--;
+
+  _hours= second_diff % DRIZZLE_SECONDS_IN_HOUR;
+  second_diff-= (_hours * DRIZZLE_SECONDS_IN_HOUR);
+  _minutes= second_diff % DRIZZLE_SECONDS_IN_MINUTE;
+  second_diff-= (_minutes * DRIZZLE_SECONDS_IN_MINUTE);
+  _seconds= second_diff;
+
+  /* Handle the microsecond precision */
+  int64_t microsecond_diff= _useconds - rhs._useconds;
+  if (microsecond_diff < 0)
+  {
+    microsecond_diff= (-1 * microsecond_diff);
+    _seconds--;
+  }
+  _useconds= microsecond_diff;
+
+  return *this;
+}
+DateTime& DateTime::operator+=(const DateTime &rhs)
+{
+  /* 
+   * Figure out the new Julian Day Number by adding the JDNs of both
+   * dates together.
+   */
+  int64_t day_left= julian_day_number_from_gregorian_date(_years, _months, _days);
+  int64_t day_right= julian_day_number_from_gregorian_date(rhs._years, rhs._months, rhs._days);
+  int64_t day_diff= day_left + day_right;
+
+  /** @TODO Need an exception check here for bounds of JDN... */
+
+  /* Now re-compose the Date's structure from the ng Julian Day Number */
+  gregorian_date_from_julian_day_number(day_diff, &_years, &_months, &_days);
+
+  /* And now handle the time components */
+  int64_t second_diff= _cumulative_seconds_in_time() + rhs._cumulative_seconds_in_time();
+
+  /* 
+   * The resulting seconds might be more than a day.  If do, 
+   * adjust our ng days up 1.
+   */
+  if (second_diff >= DRIZZLE_SECONDS_IN_DAY)
+  {
+    _days++;
+    second_diff-= (_days * DRIZZLE_SECONDS_IN_DAY);
+  }
+
+  _hours= second_diff % DRIZZLE_SECONDS_IN_HOUR;
+  second_diff-= (_hours * DRIZZLE_SECONDS_IN_HOUR);
+  _minutes= second_diff % DRIZZLE_SECONDS_IN_MINUTE;
+  second_diff-= (_minutes * DRIZZLE_SECONDS_IN_MINUTE);
+  _seconds= second_diff;
+
+  /* Handle the microsecond precision */
+  int64_t microsecond_diff= _useconds - rhs._useconds;
+  if (microsecond_diff < 0)
+  {
+    microsecond_diff= (-1 * microsecond_diff);
+    _seconds--;
+  }
+  _useconds= microsecond_diff;
+
+  return *this;
+}
+
+/*
+ * Comparison operators between two Timestamps
+ */
+bool Timestamp::operator==(const Timestamp& rhs)
+{
+  return (_epoch_seconds == rhs._epoch_seconds);
+}
+bool Timestamp::operator!=(const Timestamp& rhs)
+{
+  return ! (*this == rhs);
+}
+bool Timestamp::operator<(const Timestamp& rhs)
+{
+  return (_epoch_seconds < rhs._epoch_seconds);
+}
+bool Timestamp::operator<=(const Timestamp& rhs)
+{
+  return (_epoch_seconds <= rhs._epoch_seconds);
+}
+bool Timestamp::operator>(const Timestamp& rhs)
+{
+  return ! (*this < rhs);
+}
+bool Timestamp::operator>=(const Timestamp& rhs)
+{
+  return ! (*this <= rhs);
+}
+
+
+bool Time::from_string(const char *from, size_t from_len)
+{
+  /* 
+   * Loop through the known time formats and see if 
+   * there is a match.
+   */
+  bool matched= false;
+  TemporalFormat *current_format;
+  std::vector<TemporalFormat *>::iterator current= known_time_formats.begin();
+
+  while (current != known_time_formats.end())
+  {
+    current_format= *current;
+    if (current_format->matches(from, from_len, this))
+    {
+      matched= true;
+      break;
+    }
+    current++;
+  }
+
+  if (! matched)
+    return false;
+  else
+    return is_valid();
+}
+
+void Time::to_string(char *to, size_t *to_len) const
+{
+  *to_len= sprintf(to
+                 , "%02" PRIu32 ":%02" PRIu32 ":%02" PRIu32
+                 , _hours
+                 , _minutes
+                 , _seconds);
+}
+
+void Date::to_string(char *to, size_t *to_len) const
+{
+  *to_len= sprintf(to
+                 , "%04" PRIu32 "-%02" PRIu32 "-%02" PRIu32
+                 , _years
+                 , _months
+                 , _days);
+}
+
+void DateTime::to_string(char *to, size_t *to_len) const
+{
+  *to_len= sprintf(to
+                 , "%04" PRIu32 "-%02" PRIu32 "-%02" PRIu32 " %02" PRIu32 ":%02" PRIu32 ":%02" PRIu32
+                 , _years
+                 , _months
+                 , _days
+                 , _hours
+                 , _minutes
+                 , _seconds);
+}
+
+void MicroTimestamp::to_string(char *to, size_t *to_len) const
+{
+  *to_len= sprintf(to
+                 , "%04" PRIu32 "-%02" PRIu32 "-%02" PRIu32 " %02" PRIu32 ":%02" PRIu32 ":%02" PRIu32 ".%06" PRIu32
+                 , _years
+                 , _months
+                 , _days
+                 , _hours
+                 , _minutes
+                 , _seconds
+                 , _useconds);
+}
+
+void Time::to_decimal(my_decimal *to) const
+{
+  int64_t time_portion= (((_hours * 100L) + _minutes) * 100L) + _seconds;
+  (void) int2my_decimal(E_DEC_FATAL_ERROR, time_portion, false, to);
+  if (_useconds > 0)
+  {
+    to->buf[(to->intg-1) / 9 + 1]= _useconds * 1000;
+    to->frac= 6;
+  }
+}
+
+void Date::to_decimal(my_decimal *to) const
+{
+  int64_t date_portion= (((_years * 100L) + _months) * 100L) + _days;
+  (void) int2my_decimal(E_DEC_FATAL_ERROR, date_portion, false, to);
+}
+
+void DateTime::to_decimal(my_decimal *to) const
+{
+  int64_t date_portion= (((_years * 100L) + _months) * 100L) + _days;
+  int64_t time_portion= (((((date_portion * 100L) + _hours) * 100L) + _minutes) * 100L) + _seconds;
+  (void) int2my_decimal(E_DEC_FATAL_ERROR, time_portion, false, to);
+  if (_useconds > 0)
+  {
+    to->buf[(to->intg-1) / 9 + 1]= _useconds * 1000;
+    to->frac= 6;
+  }
+}
+
+void Date::to_int64_t(int64_t *to) const
+{
+  *to= (_years * INT32_C(10000)) 
+     + (_months * INT32_C(100)) 
+     + _days;
+}
+
+void Date::to_int32_t(int32_t *to) const
+{
+  *to= (_years * INT32_C(10000)) 
+     + (_months * INT32_C(100)) 
+     + _days;
+}
+
+void Time::to_int32_t(int32_t *to) const
+{
+  *to= (_hours * INT32_C(10000)) 
+     + (_minutes * INT32_C(100)) 
+     + _seconds;
+}
+
+void DateTime::to_int64_t(int64_t *to) const
+{
+  *to= ((
+       (_years * INT64_C(10000)) 
+     + (_months * INT64_C(100)) 
+     + _days
+       ) * INT64_C(1000000))
+     + (
+       (_hours * INT64_C(10000)) 
+     + (_minutes * INT64_C(100) )
+     + _seconds
+     );
+}
+
+void Date::to_tm(struct tm *to) const
+{
+  to->tm_sec= 0;
+  to->tm_min= 0;
+  to->tm_hour= 0;
+  to->tm_mday= _days; /* Drizzle format uses ordinal, standard tm does too! */
+  to->tm_mon= _months - 1; /* Drizzle format uses ordinal, standard tm does NOT! */
+  to->tm_year= _years - 1900;
+}
+
+void DateTime::to_tm(struct tm *to) const
+{
+  to->tm_sec= _seconds;
+  to->tm_min= _minutes;
+  to->tm_hour= _hours;
+  to->tm_mday= _days; /* Drizzle format uses ordinal, standard tm does too! */
+  to->tm_mon= _months - 1; /* Drizzle format uses ordinal, standard tm does NOT! */
+  to->tm_year= _years - 1900;
+}
+
+/**
+ * We assume that the supplied integer is an
+ * absolute day number (for instance, from the FROM_DAYS()
+ * SQL function.
+ */
+bool Date::from_julian_day_number(const int64_t from)
+{
+  /* 
+   * First convert the absolute day number into a Julian
+   * Day Number and then run a standard conversion on the
+   * JDN.
+   */
+  gregorian_date_from_julian_day_number(from, &_years, &_months, &_days);
+  return is_valid();
+}
+
+/**
+ * Ignore overflow and pass-through to DateTime::from_int64_t()
+ */
+bool Date::from_int32_t(const int32_t from)
+{
+  return ((DateTime *) this)->from_int64_t((int64_t) from);
+}
+
+/**
+ * Attempt to interpret the supplied 4-byte integer as
+ * a TIME value in the format HHmmSS
+ */
+bool Time::from_int32_t(const int32_t from)
+{
+  int32_t copy_from= from;
+  _hours= copy_from % INT32_C(10000);
+  _minutes= copy_from % INT32_C(100);
+  _seconds= copy_from & 3; /* Masks off all but last 2 digits */
+  return is_valid();
+}
+
+/**
+ * We try to intepret the incoming number as a datetime "string".
+ * This is pretty much a hack for usability, but keeps us compatible
+ * with MySQL.
+ */
+bool DateTime::from_int64_t(const int64_t from)
+{
+  int64_t copy_from= from;
+  int64_t part1;
+  int64_t part2;
+
+  if (! (copy_from == 0LL || copy_from >= 10000101000000LL))
+  {
+    if (copy_from < 101)
+      return false;
+    if (copy_from <= (DRIZZLE_YY_PART_YEAR-1)*10000L+1231L)
+      copy_from= (copy_from+20000000L)*1000000L;                 /* YYMMDD, year: 2000-2069 */
+    if (copy_from < (DRIZZLE_YY_PART_YEAR)*10000L+101L)
+      return false;
+    if (copy_from <= 991231L)
+      copy_from= (copy_from+19000000L)*1000000L;                 /* YYMMDD, year: 1970-1999 */
+    if (copy_from < 10000101L)
+      return false;
+    if (copy_from <= 99991231L)
+      copy_from= copy_from*1000000L;
+    if (copy_from < 101000000L)
+      return false;
+    if (copy_from <= (DRIZZLE_YY_PART_YEAR-1) * 10000000000LL + 1231235959LL)
+      copy_from= copy_from + 20000000000000LL;                   /* YYMMDDHHMMSS, 2000-2069 */
+    if (copy_from <  DRIZZLE_YY_PART_YEAR * 10000000000LL + 101000000LL)
+      return false;
+    if (copy_from <= 991231235959LL)
+      copy_from= copy_from + 19000000000000LL;		/* YYMMDDHHMMSS, 1970-1999 */
+  }
+
+  part1= (int64_t) (copy_from / 1000000LL);
+  part2= (int64_t) (copy_from - (int64_t) part1 * 1000000LL);
+  _years=  (uint32_t) (part1/10000L);  
+  
+  part1%=10000L;
+  _months= (uint32_t) part1 / 100;
+  _days=   (uint32_t) part1 % 100;
+  _hours=  (uint32_t) (part2/10000L);  
+
+  part2%=10000L;
+  _minutes= (uint32_t) part2 / 100;
+  _seconds= (uint32_t) part2 % 100;
+
+  set_epoch_seconds();
+  return is_valid();
+}
+
+bool Date::in_unix_epoch() const
+{
+  return in_unix_epoch_range(_years, _months, _days, 0, 0, 0);
+}
+
+bool DateTime::in_unix_epoch() const
+{
+  return in_unix_epoch_range(_years, _months, _days, _hours, _minutes, _seconds);
+}
+
+bool Date::from_tm(const struct tm *from)
+{
+  _years= 1900 + from->tm_year;
+  _months= 1 + from->tm_mon; /* Month is NOT ordinal for struct tm! */
+  _days= from->tm_mday; /* Day IS ordinal for struct tm */
+  _hours= from->tm_hour;
+  _minutes= from->tm_min;
+  _seconds= from->tm_sec;
+  /* Set hires precision to zero */
+  _useconds= 0;
+  _nseconds= 0;
+
+  set_epoch_seconds();
+  return is_valid();
+}
+
+bool Date::from_time_t(const time_t from)
+{
+  struct tm broken_time;
+  struct tm *result;
+
+  result= gmtime_r(&from, &broken_time);
+  if (result != NULL)
+  {
+    _years= 1900 + broken_time.tm_year;
+    _months= 1 + broken_time.tm_mon; /* Month is NOT ordinal for struct tm! */
+    _days= broken_time.tm_mday; /* Day IS ordinal for struct tm */
+    _hours= 0;
+    _minutes= 0;
+    _seconds= 0;
+    _epoch_seconds= 0;
+    /* Set hires precision to zero */
+    _useconds= 0;
+    _nseconds= 0;
+    return is_valid();
+  }
+  else 
+    return false;
+}
+
+bool DateTime::from_time_t(const time_t from)
+{
+  struct tm broken_time;
+  struct tm *result;
+
+  result= gmtime_r(&from, &broken_time);
+  if (result != NULL)
+  {
+    _years= 1900 + broken_time.tm_year;
+    _months= 1 + broken_time.tm_mon; /* Month is NOT ordinal for struct tm! */
+    _days= broken_time.tm_mday; /* Day IS ordinal for struct tm */
+    _hours= broken_time.tm_hour;
+    _minutes= broken_time.tm_min;
+    _seconds= broken_time.tm_sec;
+    _epoch_seconds= from;
+    /* Set hires precision to zero */
+    _useconds= 0;
+    _nseconds= 0;
+    return is_valid();
+  }
+  else 
+    return false;
+}
+
+void Date::to_time_t(time_t *to) const
+{
+  if (in_unix_epoch())
+    *to= _epoch_seconds;
+  *to= 0;
+}
+
+void Timestamp::to_time_t(time_t *to) const
+{
+  *to= _epoch_seconds;
+}
+
+void MicroTimestamp::to_timeval(struct timeval *to) const
+{
+  to->tv_sec= _epoch_seconds;
+  to->tv_usec= _useconds;
+}
+
+void NanoTimestamp::to_timespec(struct timespec *to) const
+{
+  to->tv_sec= _epoch_seconds;
+  to->tv_nsec= _nseconds;
+}
+
+bool Date::is_valid() const
+{
+  return (_years >= DRIZZLE_MIN_YEARS_SQL && _years <= DRIZZLE_MAX_YEARS_SQL)
+      && (_months >= 1 && _months <= 12)
+      && (_days >= 1 && _days <= days_in_gregorian_year_month(_years, _months));
+}
+
+bool Time::is_valid() const
+{
+  return (_years == 0)
+      && (_months == 0)
+      && (_days == 0)
+      && (_hours <= 23)
+      && (_minutes <= 59)
+      && (_seconds <= 59); /* No Leap second... TIME is for elapsed time... */
+}
+
+bool DateTime::is_valid() const
+{
+  return (_years >= DRIZZLE_MIN_YEARS_SQL && _years <= DRIZZLE_MAX_YEARS_SQL)
+      && (_months >= 1 && _months <= 12)
+      && (_days >= 1 && _days <= days_in_gregorian_year_month(_years, _months))
+      && (_hours <= 23)
+      && (_minutes <= 59)
+      && (_seconds <= 61); /* Leap second... */
+}
+
+bool Timestamp::is_valid() const
+{
+  return in_unix_epoch_range(_years, _months, _days, _hours, _minutes, _seconds);
+}
+
+bool MicroTimestamp::is_valid() const
+{
+  return in_unix_epoch_range(_years, _months, _days, _hours, _minutes, _seconds)
+      && (_useconds <= UINT32_C(999999));
+}
+
+bool NanoTimestamp::is_valid() const
+{
+  return in_unix_epoch_range(_years, _months, _days, _hours, _minutes, _seconds)
+      && (_useconds <= UINT32_C(999999))
+      && (_nseconds <= UINT32_C(999999999));
+}
+
+} /* end namespace drizzled */

=== added file 'drizzled/temporal.h'
--- drizzled/temporal.h	1970-01-01 00:00:00 +0000
+++ drizzled/temporal.h	2009-01-28 02:39:29 +0000
@@ -0,0 +1,634 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  Authors:
+ *
+ *  Jay Pipes <jay.pipes@sun.com>
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *
+ * Defines the API for dealing with temporal data inside the server.
+ *
+ * The Temporal class is the base class for all data of any temporal
+ * type.  A number of derived classes define specialized classes 
+ * representng various date, date-time, time, or timestamp types.
+ *
+ * All Temporal derived classes are ValueObjects.  That is to say that
+ * Temporal class instances are not part of the Item hierarchy and serve
+ * <em>only</em> to represent a time or date-related piece of data.
+ *
+ * @note
+ *
+ * Low-level calendrical calculations are done via routines in the
+ * calendar.cc file.
+ *
+ * @see drizzled/calendar.cc
+ */
+
+#ifndef DRIZZLED_TEMPORAL_H
+#define DRIZZLED_TEMPORAL_H
+
+#define DRIZZLE_MAX_SECONDS 59
+#define DRIZZLE_MAX_MINUTES 59
+#define DRIZZLE_MAX_HOURS 23
+#define DRIZZLE_MAX_DAYS 31
+#define DRIZZLE_MAX_MONTHS 12
+#define DRIZZLE_MAX_YEARS_SQL 9999
+#define DRIZZLE_MAX_YEARS_EPOCH 2038
+#define DRIZZLE_MIN_SECONDS 0
+#define DRIZZLE_MIN_MINUTES 0
+#define DRIZZLE_MIN_HOURS 0
+#define DRIZZLE_MIN_DAYS 1
+#define DRIZZLE_MIN_MONTHS 1
+#define DRIZZLE_MIN_YEARS_SQL 1
+#define DRIZZLE_MIN_YEARS_EPOCH 1970
+
+#define DRIZZLE_SECONDS_IN_MINUTE 60
+#define DRIZZLE_SECONDS_IN_HOUR (60*60)
+#define DRIZZLE_SECONDS_IN_DAY (60*60*24)
+#define DRIZZLE_NANOSECONDS_IN_MICROSECOND 1000
+
+#define DRIZZLE_YY_PART_YEAR  70
+
+#include <sys/time.h>
+#include <time.h>
+
+#include "drizzled/global.h"
+#include "drizzled/calendar.h"
+
+/* Outside forward declarations */
+class my_decimal;
+
+namespace drizzled 
+{
+
+/**
+ * Base class for all temporal data classes.
+ */
+class Temporal
+{
+protected:
+  enum calendar _calendar;
+  uint32_t _years;
+  uint32_t _months;
+  uint32_t _days;
+  uint32_t _hours;
+  uint32_t _minutes;
+  uint32_t _seconds;
+  time_t _epoch_seconds;
+  uint32_t _useconds;
+  uint32_t _nseconds;
+  /** Returns number of seconds in time components (hour + minute + second) */
+  uint64_t _cumulative_seconds_in_time() const;
+public:
+  Temporal();
+  virtual ~Temporal() {}
+
+  /** Returns the calendar component. */
+  inline enum calendar calendar() const {return _calendar;}
+  /** Returns the nanoseconds component. */
+  inline uint32_t nseconds() const {return _nseconds;}
+  /** Sets the useconds component. */
+  inline void set_useconds(const uint32_t usecond) {_useconds= usecond;}
+  /** Returns the microsseconds component. */
+  inline uint32_t useconds() const {return _useconds;}
+  /** Sets the epoch_seconds component. */
+  virtual void set_epoch_seconds();
+  /** Returns the UNIX epoch seconds component. */
+  inline time_t epoch_seconds() const {return _epoch_seconds;}
+  /** Sets the seconds component. */
+  inline void set_seconds(const uint32_t second) {_seconds= second;}
+  /** Returns the seconds component. */
+  inline uint32_t seconds() const {return _seconds;}
+  /** Sets the days component. */
+  inline void set_minutes(const uint32_t minute) {_minutes= minute;}
+  /** Returns the minutes component. */
+  inline uint32_t minutes() const {return _minutes;}
+  /** Sets the hours component. */
+  inline void set_hours(const uint32_t hour) {_hours= hour;}
+  /** Returns the hours component. */
+  inline uint32_t hours() const {return _hours;}
+  /** Sets the days component. */
+  inline void set_days(const uint32_t day) {_days= day;}
+  /** Returns the days component. */
+  inline uint32_t days() const {return _days;}
+  /** Sets the months component. */
+  inline void set_months(const uint32_t month) {_months= month;}
+  /** Returns the months component. */
+  inline uint32_t months() const {return _months;}
+  /** Sets the years component. */
+  inline void set_years(const uint32_t year) {_years= year;}
+  /** Returns the years component. */
+  inline uint32_t years() const {return _years;}
+
+  /** Returns whether the temporal value is valid as a date. */
+  virtual bool is_valid_date() const= 0;
+  /** Returns whether the temporal value is valid as a datetime. */
+  virtual bool is_valid_datetime() const= 0;
+  /** Returns whether the temporal value is valid as a time. */
+  virtual bool is_valid_time() const= 0;
+  /** Returns whether the temporal value is valid as a UNIX timestamp. */
+  virtual bool is_valid_timestamp() const= 0;
+
+  /**
+   * Returns whether the temporal
+   * value is valid. Each subclass defines what is
+   * valid for the range of temporal data it contains.
+   */
+  virtual bool is_valid() const= 0;
+
+  /**
+   * All Temporal derived classes must implement
+   * conversion routines for converting to and from
+   * a string. Subclasses implement other conversion 
+   * routines, but should always follow these notes:
+   *
+   * 1) Ensure that ALL from_xxx methods call is_valid()
+   * 2) Ensure that ALL to_xxx methods are void returns and
+   *    do not call is_valid()
+   *
+   * This minimizes the repeated bounds-checking to
+   * just the conversion from_xxx routines.
+   */
+
+  friend class TemporalFormat;
+};
+
+/**
+ * Class representing temporal components in a valid
+ * SQL date range, with no time component
+ */
+class Date: public Temporal 
+{
+public:
+  /**
+   * Comparison operator overloads to compare a Date against
+   * another Date value.
+   *
+   * @param Date to compare against.
+   */
+  bool operator==(const Date &rhs);
+  bool operator!=(const Date &rhs);
+  bool operator>(const Date &rhs);
+  bool operator>=(const Date &rhs);
+  bool operator<(const Date &rhs);
+  bool operator<=(const Date &rhs);
+  /**
+   * Operator overload for adding/subtracting another Date
+   * (or subclass) to/from this temporal.  When subtracting 
+   * or adding two Dates, we return a new Date instance.
+   *
+   * @param Temporal instance to add/subtract to/from
+   */
+  const Date operator-(const Date &rhs);
+  const Date operator+(const Date &rhs);
+  Date& operator+=(const Date &rhs);
+  Date& operator-=(const Date &rhs);
+
+  virtual bool is_valid_date() const {return is_valid();}
+  virtual bool is_valid_datetime() const {return is_valid();}
+  virtual bool is_valid_time() const {return false;}
+  virtual bool is_valid_timestamp() const {return is_valid() && in_unix_epoch();}
+  /** Returns whether the temporal value is valid date. */
+  virtual bool is_valid() const;
+  /* Returns whether the Date (or subclass) instance is in the Unix Epoch. */
+  virtual bool in_unix_epoch() const;
+
+  /**
+   * Fills a supplied char string with a
+   * string representation of the Date  
+   * value.
+   *
+   * @param C-String to fill.
+   * @param Length of filled string (out param)
+   */
+  virtual void to_string(char *to, size_t *to_len) const;
+
+  /**
+   * Attempts to populate the Date instance based
+   * on the contents of a supplied string.
+   *
+   * Returns whether the conversion was 
+   * successful.
+   *
+   * @param String to convert from
+   * @param Length of supplied string
+   */
+  virtual bool from_string(const char *from, size_t from_len);
+
+  /**
+   * Fills a supplied 8-byte integer pointer with an
+   * integer representation of the Date
+   * value.
+   *
+   * @param Integer to fill.
+   */
+  virtual void to_int64_t(int64_t *to) const;
+
+  /**
+   * Fills a supplied 4-byte integer pointer with an
+   * integer representation of the Date
+   * value.
+   *
+   * @param Integer to fill.
+   */
+  virtual void to_int32_t(int32_t *to) const;
+
+  /**
+   * Attempts to populate the Date instance based
+   * on the contents of a supplied 4-byte integer.
+   *
+   * Returns whether the conversion was 
+   * successful.
+   *
+   * @param Integer to convert from
+   */
+  virtual bool from_int32_t(const int32_t from);
+
+  /**
+   * Attempts to populate the Date instance based
+   * on the contents of a supplied Julian Day Number
+   *
+   * Returns whether the conversion was 
+   * successful.
+   *
+   * @param Integer to convert from
+   */
+  virtual bool from_julian_day_number(const int64_t from);
+
+  /**
+   * Fills a supplied tm pointer with an
+   * representation of the Date 
+   * value.
+   *
+   * @param tm to fill.
+   */
+  virtual void to_tm(struct tm *to) const;
+
+  /**
+   * Attempts to populate the Date instance based
+   * on the contents of a supplied pointer to struct tm
+   * (broken time).
+   *
+   * Returns whether the conversion was 
+   * successful.
+   *
+   * @param Pointe rto the struct tm to convert from
+   */
+  virtual bool from_tm(const struct tm *from);
+
+  /**
+   * Attempts to convert the Date value into
+   * a supplied time_t.
+   *
+   * @param Pointer to a time_t to convert to
+   */
+  virtual void to_time_t(time_t *to) const;
+
+  /**
+   * Attempts to populate the Date instance based
+   * on the contents of a supplied time_t
+   *
+   * Returns whether the conversion was 
+   * successful.
+   *
+   * @param time_t to convert from
+   */
+  virtual bool from_time_t(const time_t from);
+
+  /**
+   * Fills a supplied my_decimal with a representation of 
+   * the Date value.
+   *
+   * @param Pointer to the my_decimal to fill
+   */
+  virtual void to_decimal(my_decimal *to) const;
+};
+
+/* Forward declare needed for friendship */
+class DateTime;
+
+/**
+ * Class representing temporal components having only
+ * a time component, with no date structure
+ */
+class Time: public Temporal
+{
+public:
+
+  /**
+   * Comparison operator overloads to compare a Time against
+   * another Time value.
+   *
+   * @param Time to compare against.
+   */
+  bool operator==(const Time &rhs);
+  bool operator!=(const Time &rhs);
+  bool operator>(const Time &rhs);
+  bool operator>=(const Time &rhs);
+  bool operator<(const Time &rhs);
+  bool operator<=(const Time &rhs);
+  /**
+   * Operator to add/subtract a Time from a Time.  
+   * We can return a Time new temporal instance.
+   *
+   * @param Temporal instance to add/subtract to/from
+   */
+  const Time operator-(const Time &rhs);
+  const Time operator+(const Time &rhs);
+  Time& operator-=(const Time &rhs);
+  Time& operator+=(const Time &rhs);
+
+  virtual bool is_valid_date() const {return false;}
+  virtual bool is_valid_datetime() const {return false;}
+  virtual bool is_valid_time() const {return is_valid();}
+  virtual bool is_valid_timestamp() const {return false;}
+  /** Returns whether the temporal value is valid date. */
+  virtual bool is_valid() const;
+
+  /**
+   * Fills a supplied char string with a
+   * string representation of the Time  
+   * value.
+   *
+   * @param C-String to fill.
+   * @param Length of filled string (out param)
+   */
+  virtual void to_string(char *to, size_t *to_len) const;
+
+  /**
+   * Attempts to populate the Time instance based
+   * on the contents of a supplied string.
+   *
+   * Returns whether the conversion was 
+   * successful.
+   *
+   * @param String to convert from
+   * @param Length of supplied string
+   */
+  virtual bool from_string(const char *from, size_t from_len);
+
+  /**
+   * Fills a supplied 4-byte integer pointer with an
+   * integer representation of the Time
+   * value.
+   *
+   * @param Integer to fill.
+   */
+  virtual void to_int32_t(int32_t *to) const;
+
+  /**
+   * Attempts to populate the Time instance based
+   * on the contents of a supplied 4-byte integer.
+   *
+   * Returns whether the conversion was 
+   * successful.
+   *
+   * @param Integer to convert from
+   */
+  virtual bool from_int32_t(const int32_t from);
+
+  /**
+   * Fills a supplied my_decimal with a representation of 
+   * the Time value.
+   *
+   * @param Pointer to the my_decimal to fill
+   */
+  virtual void to_decimal(my_decimal *to) const;
+
+  friend class DateTime;
+};
+
+/**
+ * Class representing temporal components in a valid
+ * SQL datetime range, including a time component
+ */
+class DateTime: public Date
+{
+public:
+  /**
+   * Comparison operator overloads to compare a DateTime against
+   * another DateTime value.
+   *
+   * @param DateTime to compare against.
+   */
+  bool operator==(const DateTime &rhs);
+  bool operator!=(const DateTime &rhs);
+  bool operator>(const DateTime &rhs);
+  bool operator>=(const DateTime &rhs);
+  bool operator<(const DateTime &rhs);
+  bool operator<=(const DateTime &rhs);
+  /**
+   * Operator to add/subtract a Time from a Time.  
+   * We can return a Time new temporal instance.
+   *
+   * @param Temporal instance to add/subtract to/from
+   */
+  const DateTime operator-(const Time &rhs);
+  const DateTime operator+(const Time &rhs);
+  DateTime& operator-=(const Time &rhs);
+  DateTime& operator+=(const Time &rhs);
+  /**
+   * Operator overload for adding/subtracting another DateTime
+   * (or subclass) to/from this temporal.  When subtracting 
+   * or adding two DateTimes, we return a new DateTime instance.
+   *
+   * @param Temporal instance to add/subtract to/from
+   */
+  const DateTime operator-(const DateTime &rhs);
+  const DateTime operator+(const DateTime &rhs);
+  DateTime& operator+=(const DateTime &rhs);
+  DateTime& operator-=(const DateTime &rhs);
+
+  /* Returns whether the DateTime (or subclass) instance is in the Unix Epoch. */
+  bool in_unix_epoch() const;
+  /** Returns whether the temporal value is valid datetime. */
+  virtual bool is_valid() const;
+
+  /**
+   * It's not possible to convert to and from a DateTime and 
+   * a 4-byte integer, so let us know if we try and do it!
+   */
+  inline void to_int32_t(int32_t *) {assert(0);}
+  inline bool from_int32_t(int32_t) {assert(0); return false;}
+
+  /**
+   * Fills a supplied char string with a
+   * string representation of the DateTime
+   * value.
+   *
+   * @param C-String to fill.
+   * @param Length of filled string (out param)
+   */
+  virtual void to_string(char *to, size_t *to_len) const;
+
+  /**
+   * Attempts to populate the DateTime instance based
+   * on the contents of a supplied string.
+   *
+   * Returns whether the conversion was 
+   * successful.
+   *
+   * @param String to convert from
+   * @param Length of supplied string
+   */
+  virtual bool from_string(const char *from, size_t from_len);
+
+  /**
+   * Fills a supplied 8-byte integer pointer with an
+   * integer representation of the DateTime
+   * value.
+   *
+   * @param Integer to fill.
+   */
+  virtual void to_int64_t(int64_t *to) const;
+
+  /**
+   * Attempts to populate the DateTime instance based
+   * on the contents of a supplied time_t
+   *
+   * Returns whether the conversion was 
+   * successful.
+   *
+   * @param time_t to convert from
+   */
+  virtual bool from_time_t(const time_t from);
+
+  /**
+   * Attempts to populate the DateTime instance based
+   * on the contents of a supplied 8-byte integer.
+   *
+   * Returns whether the conversion was 
+   * successful.
+   *
+   * @param Integer to convert from
+   */
+  virtual bool from_int64_t(const int64_t from);
+
+  /**
+   * Fills a supplied tm pointer with an
+   * representation of the DateTime
+   * value.
+   *
+   * @param tm to fill.
+   */
+  virtual void to_tm(struct tm *to) const;
+
+  /**
+   * Fills a supplied my_decimal with a representation of 
+   * the DateTime value.
+   *
+   * @param Pointer to the my_decimal to fill
+   */
+  virtual void to_decimal(my_decimal *to) const;
+};
+
+/**
+ * Class representing temporal components in the UNIX epoch
+ */
+class Timestamp: public DateTime 
+{
+public:
+  /**
+   * Comparison operator overloads to compare a Timestamp against
+   * another Timestamp value.
+   *
+   * @param Timestamp to compare against.
+   */
+  bool operator==(const Timestamp &rhs);
+  bool operator!=(const Timestamp &rhs);
+  bool operator>(const Timestamp &rhs);
+  bool operator>=(const Timestamp &rhs);
+  bool operator<(const Timestamp &rhs);
+  bool operator<=(const Timestamp &rhs);
+
+  inline bool is_valid_timestamp() const {return is_valid();}
+  /** Returns whether the temporal value is valid timestamp. */
+  virtual bool is_valid() const;
+
+  /**
+   * Attempts to convert the Timestamp value into
+   * a supplied time_t.
+   *
+   * @param Pointer to a time_t to convert to
+   */
+  virtual void to_time_t(time_t *to) const;
+};
+
+/**
+ * Class representing temporal components in the UNIX epoch
+ * with an additional microsecond component.
+ */
+class MicroTimestamp: public Timestamp
+{
+public:
+  /** Returns whether the temporal value is valid micro-timestamp. */
+  bool is_valid() const;
+
+  /**
+   * Fills a supplied char string with a
+   * string representation of the MicroTimestamp
+   * value.
+   *
+   * @param C-String to fill.
+   * @param Length of filled string (out param)
+   */
+  virtual void to_string(char *to, size_t *to_len) const;
+
+  /**
+   * Fills a supplied timeval pointer with an
+   * representation of the MicroTimestamp
+   * value.
+   *
+   * Returns whether the conversion was 
+   * successful.
+   *
+   * @param timeval to fill.
+   */
+  virtual void to_timeval(struct timeval *to) const;
+};
+
+/**
+ * Class representing temporal components in the UNIX epoch
+ * with an additional nanosecond component.
+ */
+class NanoTimestamp: public Timestamp
+{
+public:
+  /** Returns whether the temporal value is valid nano-timestamp. */
+  bool is_valid() const;
+
+  /**
+   * Fills a supplied timespec pointer with an
+   * representation of the NanoTimestamp
+   * value.
+   *
+   * Returns whether the conversion was 
+   * successful.
+   *
+   * @param timespec to fill.
+   */
+  void to_timespec(struct timespec *to) const;
+};
+
+} /* end namespace drizzled */
+
+#endif /* DRIZZLED_TEMPORAL_H */

=== added file 'drizzled/temporal_format.cc'
--- drizzled/temporal_format.cc	1970-01-01 00:00:00 +0000
+++ drizzled/temporal_format.cc	2009-01-28 02:39:29 +0000
@@ -0,0 +1,246 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  Authors:
+ *
+ *  Jay Pipes <jay.pipes@sun.com>
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *
+ * Implementation of the server's date and time string matching utility.
+ */
+
+#include <string> /** C++ string class used */
+#include <string.h>
+#include <vector>
+#include <pcre.h>
+
+#include "drizzled/global.h"
+#include "drizzled/temporal_format.h"
+#include "drizzled/temporal.h"
+
+namespace drizzled
+{
+
+  TemporalFormat::TemporalFormat(const char *pattern)
+  :
+  _pattern(pattern)
+, _error_offset(0)
+, _error(NULL)
+, _year_part_index(0)
+, _month_part_index(0)
+, _day_part_index(0)
+, _hour_part_index(0)
+, _minute_part_index(0)
+, _second_part_index(0)
+, _usecond_part_index(0)
+{
+  /* Make sure we've got no junk in the match_vector. */
+  memset(_match_vector, 0, sizeof(_match_vector));
+
+  /* Compile our regular expression */
+  _re= pcre_compile(pattern
+                    , 0 /* Default options */
+                    , &_error
+                    , &_error_offset
+                    , NULL /* Use default character table */
+                    );
+}
+
+bool TemporalFormat::matches(const char *data, size_t data_len, Temporal *to)
+{
+  if (! is_valid()) 
+    return false;
+  
+  /* Simply check the subject against the compiled regular expression */
+  int32_t result= pcre_exec(_re
+                            , NULL /* No extra data */
+                            , data
+                            , data_len
+                            , 0 /* Start at offset 0 of subject...*/
+                            , 0 /* Default options */
+                            , _match_vector
+                            , OUT_VECTOR_SIZE
+                            );
+  if (result < 0)
+  {
+    switch (result)
+    {
+      case PCRE_ERROR_NOMATCH:
+        return false; /* No match, just return false */
+      default:
+        return false;
+    }
+    return false;
+  }
+
+  int32_t expected_match_count= (_year_part_index > 1 ? 1 : 0)
+                              + (_month_part_index > 1 ? 1 : 0)
+                              + (_day_part_index > 1 ? 1 : 0)
+                              + (_hour_part_index > 1 ? 1 : 0)
+                              + (_minute_part_index > 1 ? 1 : 0)
+                              + (_second_part_index > 1 ? 1 : 0)
+                              + 1; /* Add one for the entire match... */
+  if (result != expected_match_count)
+    return false;
+
+  /* C++ string class easy to use substr() method is very useful here */
+  std::string copy_data(data, data_len);
+  /* 
+   * OK, we have the expected substring matches, so grab
+   * the various temporal parts from the subject string
+   *
+   * @note 
+   *
+   * TemporalFormatMatch is a friend class to Temporal, so
+   * we can access the temporal instance's protected data.
+   */
+  if (_year_part_index > 1)
+  {
+    size_t year_start= _match_vector[_year_part_index];
+    size_t year_len= _match_vector[_year_part_index + 1] - _match_vector[_year_part_index];
+    to->_years= atoi(copy_data.substr(year_start, year_len).c_str());
+    if (year_len == 2)
+      to->_years+= (to->_years >= DRIZZLE_YY_PART_YEAR ? 1900 : 2000);
+  }
+  if (_month_part_index > 1)
+  {
+    size_t month_start= _match_vector[_month_part_index];
+    size_t month_len= _match_vector[_month_part_index + 1] - _match_vector[_month_part_index];
+    to->_months= atoi(copy_data.substr(month_start, month_len).c_str());
+  }
+  if (_day_part_index > 1)
+  {
+    size_t day_start= _match_vector[_day_part_index];
+    size_t day_len= _match_vector[_day_part_index + 1] - _match_vector[_day_part_index];
+    to->_days= atoi(copy_data.substr(day_start, day_len).c_str());
+  }
+  if (_hour_part_index > 1)
+  {
+    size_t hour_start= _match_vector[_hour_part_index];
+    size_t hour_len= _match_vector[_hour_part_index + 1] - _match_vector[_hour_part_index];
+    to->_hours= atoi(copy_data.substr(hour_start, hour_len).c_str());
+  }
+  if (_minute_part_index > 1)
+  {
+    size_t minute_start= _match_vector[_minute_part_index];
+    size_t minute_len= _match_vector[_minute_part_index + 1] - _match_vector[_minute_part_index];
+    to->_minutes= atoi(copy_data.substr(minute_start, minute_len).c_str());
+  }
+  if (_second_part_index > 1)
+  {
+    size_t second_start= _match_vector[_second_part_index];
+    size_t second_len= _match_vector[_second_part_index + 1] - _match_vector[_second_part_index];
+    to->_seconds= atoi(copy_data.substr(second_start, second_len).c_str());
+  }
+  if (_usecond_part_index > 1)
+  {
+    size_t usecond_start= _match_vector[_usecond_part_index];
+    size_t usecond_len= _match_vector[_usecond_part_index + 1] - _match_vector[_usecond_part_index];
+    to->_useconds= atoi(copy_data.substr(usecond_start, usecond_len).c_str());
+  }
+  return true;
+}
+
+} /* end namespace drizzled */
+
+#define COUNT_KNOWN_FORMATS 12
+
+struct temporal_format_args
+{
+  const char *pattern;
+  int32_t year_part_index;
+  int32_t month_part_index;
+  int32_t day_part_index;
+  int32_t hour_part_index;
+  int32_t minute_part_index;
+  int32_t second_part_index;
+  int32_t usecond_part_index;
+};
+
+/**
+ * A collection of all known format strings.
+ *
+ * @note
+ *
+ * IMPORTANT: Make sure TIMESTAMP and DATETIME formats precede DATE formats and TIME formats, 
+ * as the matching functionality matches on the first hit.
+ *
+ * @note 
+ *
+ * Remember to increment COUNT_KNOWN_FORMATS when you add a known format!
+ */
+static struct temporal_format_args __format_args[COUNT_KNOWN_FORMATS]= 
+{
+  {"(\\d{4})(\\d{2})(\\d{2})(\\d{2})(\\d{2})(\\d{2})", 1, 2, 3, 4, 5, 6, 0} /* YYYYMMDDHHmmSS */
+, {"(\\d{4})(\\d{2})(\\d{2})(\\d{2})(\\d{2})(\\d{2})\\.(\\d{6})", 1, 2, 3, 4, 5, 6, 7} /* YYYYMMDDHHmmSS.uuuuuu */
+, {"(\\d{4})[-/.](\\d{1,2})[-/.](\\d{1,2})[T|\\s+](\\d{2}):(\\d{2}):(\\d{2})", 1, 2, 3, 4, 5, 6, 0} /* YYYY-MM-DD[T]HH:mm:SS, YYYY.MM.DD[T]HH:mm:SS, YYYY/MM/DD[T]HH:mm:SS*/
+, {"(\\d{4})[-/.](\\d{1,2})[-/.](\\d{1,2})", 1, 2, 3, 0, 0, 0, 0} /* YYYY-MM-DD, YYYY.MM.DD, YYYY/MM/DD */
+, {"(\\d{4})(\\d{2})(\\d{2})", 1, 2, 3, 0, 0, 0, 0} /* YYYYMMDD */
+, {"(\\d{2})[-/.]*(\\d{2})[-/.]*(\\d{4})", 3, 1, 2, 0, 0, 0, 0} /* MM[-/.]DD[-/.]YYYY (US common format)*/
+, {"(\\d{2})[-/.]*(\\d{2})[-/.]*(\\d{2})", 1, 2, 3, 0, 0, 0, 0} /* YY[-/.]MM[-/.]DD */
+, {"(\\d{2})[-/.]*(\\d{1,2})[-/.]*(\\d{1,2})", 1, 2, 3, 0, 0, 0, 0} /* YY[-/.][M]M[-/.][D]D */
+, {"(\\d{2}):*(\\d{2}):*(\\d{2})\\.(\\d{6})", 0, 0, 0, 1, 2, 3, 4} /* HHmmSS.uuuuuu, HH:mm:SS.uuuuuu */
+, {"(\\d{1,2}):*(\\d{2}):*(\\d{2})", 0, 0, 0, 1, 2, 3, 0} /* [H]HmmSS, [H]H:mm:SS */
+, {"(\\d{1,2}):*(\\d{2})", 0, 0, 0, 0, 1, 2, 0} /* [m]mSS, [m]m:SS */
+, {"(\\d{1,2})", 0, 0, 0, 0, 0, 1, 0} /* SS, S */
+};
+
+std::vector<drizzled::TemporalFormat*> known_datetime_formats;
+std::vector<drizzled::TemporalFormat*> known_date_formats;
+std::vector<drizzled::TemporalFormat*> known_time_formats;
+
+/**
+ * We allocate and initialize all known date/time formats.
+ *
+ * @TODO Cut down calls to new. Allocate as a block...
+ */
+bool init_temporal_formats()
+{
+  /* Compile all the regular expressions for the datetime formats */
+  drizzled::TemporalFormat *tmp;
+  struct temporal_format_args current_format_args;
+  int32_t x;
+  
+  for (x= 0; x<COUNT_KNOWN_FORMATS; ++x)
+  {
+    current_format_args= __format_args[x];
+    tmp= new drizzled::TemporalFormat(current_format_args.pattern);
+    tmp->set_year_part_index(current_format_args.year_part_index);
+    tmp->set_month_part_index(current_format_args.month_part_index);
+    tmp->set_day_part_index(current_format_args.day_part_index);
+    tmp->set_hour_part_index(current_format_args.hour_part_index);
+    tmp->set_minute_part_index(current_format_args.minute_part_index);
+    tmp->set_second_part_index(current_format_args.second_part_index);
+    tmp->set_usecond_part_index(current_format_args.usecond_part_index);
+
+    if (current_format_args.year_part_index > 0)
+    {
+      known_datetime_formats.push_back(tmp);
+      if (current_format_args.hour_part_index == 0)
+        known_date_formats.push_back(tmp);
+    }
+    if (current_format_args.hour_part_index > 0)
+      if (current_format_args.year_part_index == 0)
+        known_time_formats.push_back(tmp);
+  }
+  return true;
+}

=== added file 'drizzled/temporal_format.h'
--- drizzled/temporal_format.h	1970-01-01 00:00:00 +0000
+++ drizzled/temporal_format.h	2009-01-28 02:39:29 +0000
@@ -0,0 +1,142 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008 Sun Microsystems
+ *
+ *  Authors:
+ *
+ *  Jay Pipes <jay.pipes@sun.com>
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * @file 
+ *
+ * Defines the API for matching datetime formats.
+ */
+
+#ifndef DRIZZLED_TEMPORAL_FORMAT_MATCH_H
+#define DRIZZLED_TEMPORAL_FORMAT_MATCH_H
+
+#include <pcre.h>
+
+/* Output vector size for pcre matching.  Should be multiple of 3. */
+#define OUT_VECTOR_SIZE 30
+
+namespace drizzled
+{
+
+/* Forward declaration needed */
+class Temporal;
+
+class TemporalFormat
+{
+protected:
+  const char *_pattern; /**< The regular expression string to match */
+  pcre *_re; /**< The compiled regular expression struct */
+  int32_t _error_offset; /**< Any error encountered during compilation or matching */
+  const char *_error;
+  int32_t _match_vector[OUT_VECTOR_SIZE]; /**< Stores match substring indexes */
+  /* Index of the pattern which is a specific temporal part */
+  uint32_t _year_part_index;
+  uint32_t _month_part_index;
+  uint32_t _day_part_index;
+  uint32_t _hour_part_index;
+  uint32_t _minute_part_index;
+  uint32_t _second_part_index;
+  uint32_t _usecond_part_index;
+public:
+  /**
+   * Constructor which takes a regex string as
+   * it's only parameter.
+   *
+   * @param Pattern to use in matching
+   */
+  TemporalFormat(const char *pattern);
+  /**
+   * Returns whether the instance is compiled
+   * and contains a valid regular expression.
+   */
+  inline bool is_valid() const {return _re && (_error == NULL);}
+  /**
+   * Sets the index for the year part of the pattern.
+   *
+   * @param index of the temporal part
+   */
+  inline void set_year_part_index(int32_t index) {_year_part_index= ((index - 1) * 2) + 2;}
+  /**
+   * Sets the index for the month part of the pattern.
+   *
+   * @param index of the temporal part
+   */
+  inline void set_month_part_index(int32_t index) {_month_part_index= ((index - 1) * 2) + 2;}
+  /**
+   * Sets the index for the day part of the pattern.
+   *
+   * @param index of the temporal part
+   */
+  inline void set_day_part_index(int32_t index) {_day_part_index= ((index - 1) * 2) + 2;}
+  /**
+   * Sets the index for the hour part of the pattern.
+   *
+   * @param index of the temporal part
+   */
+  inline void set_hour_part_index(int32_t index) {_hour_part_index= ((index - 1) * 2) + 2;}
+  /**
+   * Sets the index for the minute part of the pattern.
+   *
+   * @param index of the temporal part
+   */
+  inline void set_minute_part_index(int32_t index) {_minute_part_index= ((index - 1) * 2) + 2;}
+  /**
+   * Sets the index for the second part of the pattern.
+   *
+   * @param index of the temporal part
+   */
+  inline void set_second_part_index(int32_t index) {_second_part_index= ((index - 1) * 2) + 2;}
+  /**
+   * Sets the index for the second part of the pattern.
+   *
+   * @param index of the temporal part
+   */
+  inline void set_usecond_part_index(int32_t index) {_usecond_part_index= ((index - 1) * 2) + 2;}
+  /**
+   * Returns true or false whether a supplied
+   * string matches the internal pattern for this
+   * temporal format string.
+   *
+   * @param Subject to match
+   * @param Length of subject string
+   */
+  bool matches(const char *data, size_t data_len, Temporal *to);
+};
+
+} /* end namespace drizzled */
+
+/**
+ * Initializes the regular expressions used by the datetime
+ * string matching conversion functions.
+ *
+ * Returns whether initialization was successful.
+ *
+ * @note
+ *
+ * This function is not thread-safe.  Call before threading
+ * is initialized on server init.
+ */
+bool init_temporal_formats();
+
+#endif /* DRIZZLED_TEMPORAL_FORMAT_MATCH_H */

=== modified file 'tests/r/func_time.result'
--- tests/r/func_time.result	2008-12-06 22:41:03 +0000
+++ tests/r/func_time.result	2009-01-28 02:39:29 +0000
@@ -42,8 +42,8 @@
 select month("1997-01-02"),year("98-02-03"),dayofyear("1997-12-31");
 month("1997-01-02")	year("98-02-03")	dayofyear("1997-12-31")
 1	1998	365
-select month("2001-02-00"),year("2001-00-00");
-month("2001-02-00")	year("2001-00-00")
+select month("2001-02-00"),year("2001-01-01");
+month("2001-02-00")	year("2001-01-01")
 2	2001
 select DAYOFYEAR("1997-03-03"), WEEK("1998-03-03"), QUARTER(980303);
 DAYOFYEAR("1997-03-03")	WEEK("1998-03-03")	QUARTER(980303)
@@ -450,10 +450,7 @@
 Warnings:
 Warning	1292	Incorrect datetime value: ''
 SELECT year(updated) from t1;
-year(updated)
-NULL
-Warnings:
-Warning	1292	Incorrect datetime value: ''
+ERROR HY000: Received an invalid datetime value ''.
 drop table t1;
 create table t1 (d date, dt datetime, t timestamp, c char(10));
 insert into t1 values ("0000-00-00", "0000-00-00", "0000-00-00", "0000-00-00");
@@ -479,8 +476,7 @@
 Warning	1292	Incorrect datetime value: '0000-00-00'
 Warning	1292	Incorrect datetime value: '0000-00-00'
 select year("0000-00-00"),year(d),year(dt),year(t),year(c) from t1;
-year("0000-00-00")	year(d)	year(dt)	year(t)	year(c)
-0	0	0	0	0
+ERROR HY000: Received an invalid datetime value '0000-00-00'.
 select yearweek("0000-00-00"),yearweek(d),yearweek(dt),yearweek(t),yearweek(c) from t1;
 yearweek("0000-00-00")	yearweek(d)	yearweek(dt)	yearweek(t)	yearweek(c)
 NULL	NULL	NULL	NULL	NULL

=== modified file 'tests/r/type_date.result'
--- tests/r/type_date.result	2008-12-06 22:41:03 +0000
+++ tests/r/type_date.result	2009-01-28 02:39:29 +0000
@@ -118,13 +118,7 @@
 @d:=1311
 1311
 select year(@d), month(@d), day(@d), cast(@d as date);
-year(@d)	month(@d)	day(@d)	cast(@d as date)
-NULL	NULL	NULL	NULL
-Warnings:
-Warning	1292	Incorrect datetime value: '1311'
-Warning	1292	Incorrect datetime value: '1311'
-Warning	1292	Incorrect datetime value: '1311'
-Warning	1292	Incorrect datetime value: '1311'
+ERROR HY000: Received an invalid datetime value '1311'.
 create table t1 (d  date , dt datetime , ts timestamp);
 insert into t1 values (9912101,9912101,9912101);
 ERROR 22007: Incorrect date value: '9912101' for column 'd' at row 1

=== added file 'tests/r/year.result'
--- tests/r/year.result	1970-01-01 00:00:00 +0000
+++ tests/r/year.result	2009-01-28 02:39:29 +0000
@@ -0,0 +1,123 @@
+SELECT YEAR(NULL);
+YEAR(NULL)
+NULL
+SELECT YEAR();
+ERROR 42000: You have an error in your SQL syntax; check the manual that corresponds to your Drizzle server version for the right syntax to use near ')' at line 1
+SELECT YEAR(1, 0);
+ERROR 42000: You have an error in your SQL syntax; check the manual that corresponds to your Drizzle server version for the right syntax to use near ' 0)' at line 1
+SELECT YEAR("xxx");
+ERROR HY000: Received an invalid datetime value 'xxx'.
+SELECT YEAR("0000-00-00");
+ERROR HY000: Received an invalid datetime value '0000-00-00'.
+SELECT YEAR("0000-01-01");
+ERROR HY000: Received an invalid datetime value '0000-01-01'.
+SELECT YEAR("0001-00-01");
+ERROR HY000: Received an invalid datetime value '0001-00-01'.
+SELECT YEAR("0001-01-00");
+ERROR HY000: Received an invalid datetime value '0001-01-00'.
+SELECT YEAR("2000-02-30");
+ERROR HY000: Received an invalid datetime value '2000-02-30'.
+SELECT YEAR("1900-02-29");
+ERROR HY000: Received an invalid datetime value '1900-02-29'.
+SELECT YEAR('1976-15-15');
+ERROR HY000: Received an invalid datetime value '1976-15-15'.
+SELECT YEAR("2009-01-12");
+YEAR("2009-01-12")
+2009
+SELECT YEAR("70-12-31");
+YEAR("70-12-31")
+1970
+SELECT YEAR("99-12-31");
+YEAR("99-12-31")
+1999
+SELECT YEAR("69-12-31");
+YEAR("69-12-31")
+2069
+SELECT YEAR("0001-12-31");
+YEAR("0001-12-31")
+1
+SELECT YEAR("9999-12-31");
+YEAR("9999-12-31")
+9999
+SELECT YEAR('07/31/2009');
+YEAR('07/31/2009')
+2009
+SELECT YEAR('20090916');
+YEAR('20090916')
+2009
+SELECT YEAR('20090916091528');
+YEAR('20090916091528')
+2009
+SELECT YEAR("20090916091528.000000");
+YEAR("20090916091528.000000")
+2009
+SELECT YEAR('09-09-16');
+YEAR('09-09-16')
+2009
+SELECT YEAR('09.09.16');
+YEAR('09.09.16')
+2009
+SELECT YEAR('09/09/16');
+YEAR('09/09/16')
+2009
+SELECT YEAR('09-09-16');
+YEAR('09-09-16')
+2009
+SELECT YEAR('09-9-2');
+YEAR('09-9-2')
+2009
+SELECT YEAR(20090916);
+YEAR(20090916)
+2009
+SELECT YEAR(20090916091528);
+YEAR(20090916091528)
+2009
+SELECT YEAR(20090916091528.000000);
+ERROR HY000: Received an invalid datetime value '20090916091528.000000'.
+USE test;
+DROP TABLE IF EXISTS t1;
+CREATE TABLE t1 (
+int_fld INT NOT NULL
+, date_fld DATE NOT NULL
+, datetime_fld DATETIME NOT NULL
+, timestamp_fld TIMESTAMP NOT NULL
+, char_fld CHAR(22) NOT NULL
+, varchar_fld VARCHAR(22) NOT NULL
+, text_fld TEXT NOT NULL
+, blob_fld BLOB NOT NULL
+);
+INSERT INTO t1 VALUES (
+20071130
+, "2007-11-30"
+, "2007-11-30 16:30:19"
+, "2007-11-30T16:30:19"
+, "2007-11-30 16:30:19"
+, "2007-11-30 16:30:19"
+, "2007-11-30T16:30:19"
+, "2007-11-30T16:30:19"
+);
+SELECT YEAR(int_fld) FROM t1;
+YEAR(int_fld)
+2007
+SELECT YEAR(date_fld) FROM t1;
+YEAR(date_fld)
+2007
+SELECT YEAR(datetime_fld) FROM t1;
+YEAR(datetime_fld)
+2007
+SELECT YEAR(timestamp_fld) FROM t1;
+YEAR(timestamp_fld)
+2007
+SELECT YEAR(char_fld) FROM t1;
+YEAR(char_fld)
+2007
+SELECT YEAR(varchar_fld) FROM t1;
+YEAR(varchar_fld)
+2007
+SELECT YEAR(text_fld) FROM t1;
+YEAR(text_fld)
+2007
+SELECT YEAR(blob_fld) FROM t1;
+YEAR(blob_fld)
+2007
+DROP TABLE t1;

=== modified file 'tests/t/func_time.test'
--- tests/t/func_time.test	2008-12-06 22:41:03 +0000
+++ tests/t/func_time.test	2009-01-28 02:39:29 +0000
@@ -21,7 +21,7 @@
 select date_format("1997-01-02", concat("%M %W %D ","%Y %y %m %d %h %i %s %w"));
 select dayofmonth("1997-01-02"),dayofmonth(19970323);
 select month("1997-01-02"),year("98-02-03"),dayofyear("1997-12-31");
-select month("2001-02-00"),year("2001-00-00");
+select month("2001-02-00"),year("2001-01-01");
 select DAYOFYEAR("1997-03-03"), WEEK("1998-03-03"), QUARTER(980303);
 select HOUR("1997-03-03 23:03:22"), MINUTE("23:03:22"), SECOND(230322);
 
@@ -199,6 +199,7 @@
 CREATE TABLE t1 (updated text) ENGINE=MyISAM;
 INSERT INTO t1 VALUES ('');
 SELECT month(updated) from t1;
+--error 1686 # "" is not a date for pete's sake.
 SELECT year(updated) from t1;
 drop table t1;
 
@@ -214,6 +215,7 @@
 select month("0000-00-00"),month(d),month(dt),month(t),month(c) from t1;
 select quarter("0000-00-00"),quarter(d),quarter(dt),quarter(t),quarter(c) from t1;
 select week("0000-00-00"),week(d),week(dt),week(t),week(c) from t1;
+--error 1686 # Argh.  0000-00-00 is not a date.
 select year("0000-00-00"),year(d),year(dt),year(t),year(c) from t1;
 select yearweek("0000-00-00"),yearweek(d),yearweek(dt),yearweek(t),yearweek(c) from t1;
 select to_days("0000-00-00"),to_days(d),to_days(dt),to_days(t),to_days(c) from t1;

=== modified file 'tests/t/type_date.test'
--- tests/t/type_date.test	2008-12-06 22:41:03 +0000
+++ tests/t/type_date.test	2009-01-28 02:39:29 +0000
@@ -126,6 +126,7 @@
 select @d:=011111;
 select year(@d), month(@d), day(@d), cast(@d as date);
 select @d:=1311;
+--error 1686 # Invalid datetime of 1311
 select year(@d), month(@d), day(@d), cast(@d as date);
 create table t1 (d  date , dt datetime , ts timestamp);
 -- error 1292

=== added file 'tests/t/year.test'
--- tests/t/year.test	1970-01-01 00:00:00 +0000
+++ tests/t/year.test	2009-01-28 02:39:29 +0000
@@ -0,0 +1,136 @@
+#
+# Tests for the YEAR() function
+#
+# The Drizzle YEAR() function differs from the MySQL YEAR()
+# function in these ways:
+#
+#  * Does not accept invalid parameters.  This results in an error
+#    in Drizzle.
+#
+
+# YEAR() on a NULL should produce
+# a NULL.
+SELECT YEAR(NULL);
+
+# 
+# Test improper argument list 
+#
+# 1 arg is required.
+--error 1064 
+# Wrong parameter count...but unfortunately produces 1064 Syntax Error due to limitations of 
+# the SQL parser, which considers YEAR a keyword before being a function symbol
+SELECT YEAR();
+--error 1064
+# Wrong parameter count...but unfortunately produces 1064 Syntax Error due to limitations of 
+# the SQL parser, which considers YEAR a keyword before being a function symbol
+SELECT YEAR(1, 0);
+
+# 
+# Test invalid dates passed to YEAR
+# produce an error, not a NULL or anything
+# else...
+#
+--error 1686
+SELECT YEAR("xxx");
+
+# Indy, bad dates!
+--error 1686
+SELECT YEAR("0000-00-00"); # No 0000-00-00 dates!...
+--error 1686
+SELECT YEAR("0000-01-01"); # No zero year parts
+--error 1686
+SELECT YEAR("0001-00-01"); # No zero month parts
+--error 1686
+SELECT YEAR("0001-01-00"); # No zero day parts
+--error 1686
+SELECT YEAR("2000-02-30"); # No Feb 30th!
+--error 1686
+SELECT YEAR("1900-02-29"); # Not a leap YEAR since not divisible evenly by 400...
+--error 1686
+SELECT YEAR('1976-15-15'); # No 15th month!
+
+# A good date, which should output 2009
+SELECT YEAR("2009-01-12");
+
+# A good date, which should output 1970 - Test of 2 digit YEAR conversion...
+SELECT YEAR("70-12-31");
+
+# A good date, which should output 1999 - Test of 2 digit YEAR conversion...
+SELECT YEAR("99-12-31");
+
+# A good date, which should output 2069 - Test of 2 digit YEAR conversion...
+SELECT YEAR("69-12-31");
+
+# A good date, which should output 1
+SELECT YEAR("0001-12-31");
+
+# A good date, which should output 9999
+SELECT YEAR("9999-12-31");
+
+# A good date in the common USA format, should output 2007
+SELECT YEAR('07/31/2009');
+
+# 
+# The following tests should produce errors for YEAR().    
+# 
+
+#
+# Test boundary dates for YEAR 
+#
+
+# 
+# Tests for alternate DATETIME syntaxes 
+# in string and numeric formats. All should
+# return 2009.
+#
+SELECT YEAR('20090916');
+SELECT YEAR('20090916091528');
+SELECT YEAR("20090916091528.000000");
+SELECT YEAR('09-09-16');
+SELECT YEAR('09.09.16');
+SELECT YEAR('09/09/16');
+SELECT YEAR('09-09-16');
+SELECT YEAR('09-9-2');
+SELECT YEAR(20090916);
+SELECT YEAR(20090916091528);
+--error 1686 # Don't yet support DOUBLE -> DATETIME.
+SELECT YEAR(20090916091528.000000);
+
+# Now test field values of different types
+
+USE test;
+--disable_warnings
+DROP TABLE IF EXISTS t1;
+--enable_warnings
+CREATE TABLE t1 (
+  int_fld INT NOT NULL
+, date_fld DATE NOT NULL
+, datetime_fld DATETIME NOT NULL
+, timestamp_fld TIMESTAMP NOT NULL
+, char_fld CHAR(22) NOT NULL
+, varchar_fld VARCHAR(22) NOT NULL
+, text_fld TEXT NOT NULL
+, blob_fld BLOB NOT NULL
+);
+
+INSERT INTO t1 VALUES (
+20071130
+, "2007-11-30"
+, "2007-11-30 16:30:19"
+, "2007-11-30T16:30:19"
+, "2007-11-30 16:30:19"
+, "2007-11-30 16:30:19"
+, "2007-11-30T16:30:19"
+, "2007-11-30T16:30:19"
+);
+
+SELECT YEAR(int_fld) FROM t1;
+SELECT YEAR(date_fld) FROM t1;
+SELECT YEAR(datetime_fld) FROM t1;
+SELECT YEAR(timestamp_fld) FROM t1;
+SELECT YEAR(char_fld) FROM t1;
+SELECT YEAR(varchar_fld) FROM t1;
+SELECT YEAR(text_fld) FROM t1;
+SELECT YEAR(blob_fld) FROM t1;
+
+DROP TABLE t1;

